#ifndef __ATL_DOCK_H__
#define __ATL_DOCK_H__

/////////////////////////////////////////////////////////////////////////////
// atldock.h - Docking framework for the WTL library
// http://www.viksoe.dk/code/atldock.htm
//
// Written by Bjarke Viksoe (bjarke@viksoe.dk)
// Thanks to Mike Simon for adding the ability to close a view.
// Copyright (c) 2000-2002 Bjarke Viksoe.
//
// This code may be used in compiled form in any way you desire. This
// source file may be redistributed by any means PROVIDING it is 
// not sold for profit without the authors written consent, and 
// providing that this notice and the authors name is included. 
//
// This file is provided "as is" with no expressed or implied warranty.
// The author accepts no liability if it causes any damage to you or your
// computer whatsoever. It's free, so don't hassle me about it.
//
// Beware of bugs.
//

/////////////////////////////////////////////////////////////////////////////
// edited by N.A.Bozinis for consistency and better positioning code
// also registry persistence (LoadBarState)
// 01/04/2011 15:01:35 --- http://zabkat.com
//
// TODO: FUTURE IMPROVEMENTS
// * Docking sites side by side eg dock 2 full height panes on the left
// * WM_GETMINMAXINFO to setup max/min size beyond current simplistic measures
//		true dockbar functionality where panes can be fixed size (eg toolbars)
// * observe SPI_GETDRAGFULLWINDOWS and don't draw ghostbars etc
// * correct/improve all lines with @@@ marker
// * some context menu on right click?
// * mouse d-click on title: toggle float/dock; on splitter: pane equal size
// * incorporate bjarke's CAutoHide class for sliding panels
// * if docked use thinner tracking rectangle like MFC does it
//		(see C:\Program Files\Microsoft Visual Studio\VC98\MFC\SRC\DOCKCONT.CPP)
//		also check why drawing the tracker on desktop is so slow in vista/7?


#pragma once

#ifndef __cplusplus
   #error ATL requires C++ compilation (use a .cpp suffix)
#endif

#ifndef __ATLAPP_H__
   #error atldock.h requires atlapp.h to be included first
#endif


#ifdef UPRIGHT_REGISTRYACCESS_H
// use the advanced registry class if available
#define DOCK_DUAL_REGINI
#endif


// Dock positions
#define DOCK_LEFT      0
#define DOCK_TOP       1
#define DOCK_RIGHT     2
#define DOCK_BOTTOM    3
#define DOCK_FLOAT     4
#define DOCK_HIDDEN    5
#define DOCK_LASTKNOWN 6

// Extended dock styles
#define DCK_EX_NOLEFT      (1<<DOCK_LEFT) /* no left docking etc */
#define DCK_EX_NOTOP       (1<<DOCK_TOP)
#define DCK_EX_NORIGHT     (1<<DOCK_RIGHT)
#define DCK_EX_NOBOTTOM    (1<<DOCK_BOTTOM)
#define DCK_EX_DESTROYONCLOSE 0x00000010 // nop now
#define DCK_EX_REMEMBERSIZE   0x00000020

// Control style flags
#define DCK_NOLEFT      (1<<DOCK_LEFT)
#define DCK_NOTOP       (1<<DOCK_TOP)
#define DCK_NORIGHT     (1<<DOCK_RIGHT)
#define DCK_NOBOTTOM    (1<<DOCK_BOTTOM)
#define DCK_ONLYHORZ    (DCK_NOLEFT | DCK_NORIGHT)
#define DCK_ONLYVERT    (DCK_NOTOP | DCK_NOBOTTOM)
// how many edges around us?
#define DCK_EDGELEFT    (0x10<<DOCK_LEFT)
#define DCK_EDGETOP     (0x10<<DOCK_TOP)
#define DCK_EDGERIGHT   (0x10<<DOCK_RIGHT)
#define DCK_EDGEBOTTOM  (0x10<<DOCK_BOTTOM)
#define DCK_ALLEDGE     (DCK_EDGELEFT | DCK_EDGETOP | DCK_EDGERIGHT | DCK_EDGEBOTTOM)
#define DCK_NOSPLITTER  0x00000100L // E_NOTIMPL, daft anyway
#define DCK_NOFLOAT     0x00000200L
#define DCK_NOHIDE      0x00000400L /* no close button */
// styles controlling the type of docked titlebar
#define DCK_MANUALSIDE  0x00001000L /* manual titlebar orientation top or left */
#define DCK_LEFTBAR     0x00002000L /* otherwise it is top */
#define DCK_TITLETEXT   0x00004000L /* bar shows text (otherwise grippers) */
#define DCK_GRADIENTEXT 0x00008000L /* gradient background (otherwise plain text) */
#define DCK_NOGRIPPER   0x00010000L /* silly flag for no titlebar (use a splitter!) */

#define ATL_SIMPLE_DOCKVIEW_STYLE \
   (WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS)

// Docked child command chaining macro @@@ iswindowvisible overkill?
#define CHAIN_DOCK_CHILD_COMMANDS(hwnd) \
   if(uMsg == WM_COMMAND) \
   { \
      if(::IsWindowVisible(hwnd)) \
         ::SendMessage(hwnd, uMsg, wParam, lParam); \
   }

// Docking position helpers
inline bool IsDockedVertically(short Side) { return (Side == DOCK_LEFT) || (Side == DOCK_RIGHT); };
//inline bool IsDocked(short Side) { return (Side == DOCK_TOP) || (Side == DOCK_BOTTOM) || (Side == DOCK_LEFT) || (Side == DOCK_RIGHT); };
inline bool IsDocked(short Side) { return Side >= DOCK_LEFT && Side <= DOCK_BOTTOM; };
inline bool IsFloating(short Side) { return Side == DOCK_FLOAT; };

// query the size of real frame client
#define DOCK_INFO_CHILD 0x1000

// Minimum size of docking pane
#define MIN_DOCKPANE_SIZE 28
// Default width/height of docking pane
#define DEFAULT_DOCKPANE_SIZE 170
// Default size of floating window rectangle
#define DEFAULT_FLOAT_SIZE 120
// The splitter size in pixels
#define DEFAULT_SPLITTER_SIZE 4 /* like wtl splitter */

#define WM_DOCK_QUERYRECT         WM_USER+840
#define WM_DOCK_QUERYTRACK        WM_USER+841
#define WM_DOCK_UNDOCK            WM_USER+842
#define WM_DOCK_UNFLOAT           WM_USER+843
#define WM_DOCK_DOCK              WM_USER+844
#define WM_DOCK_FLOAT             WM_USER+845
#define WM_DOCK_UPDATELAYOUT      WM_USER+846
#define WM_DOCK_REPOSITIONWINDOW  WM_USER+847
#define WM_DOCK_SETSPLITTER       WM_USER+848
#define WM_DOCK_CLIENT_CLOSE      WM_USER+849
#define WM_DOCK_RESIZEPANE        WM_USER+850

class CDockingPaneChildWindow;
class CFloatingWindow;

struct DOCKCONTEXT 
{
   HWND hwndDocked;   // The docked pane
   HWND hwndFloated;  // The floating pane
   HWND hwndChild;    // The view window
   HWND hwndOrigPrnt; // The original parent window
   short Side;        // Dock state
   short LastSide;    // Last dock state
   RECT rcWindow;     // Preferred window size (client coords except if floating)
   //SIZE sizeFloat;    // Last window size (floating) ~nab: now implicit in rcWindow
   HWND hwndRoot;     // Main dock window
	UINT idDockPane;   // pane identifier in range [ATL_IDW_COMMAND_BAR, ATL_IDW_CLIENT]
   //
   DWORD dwFlags;     // Extra flags DCK_xxx
   bool bKeepSize;    // Recommend using current size and avoid rescale


	DOCKCONTEXT() {hwndDocked=0; hwndFloated=0; hwndRoot=0;}
	~DOCKCONTEXT() {
		if(IsWindow(hwndDocked))
			DestroyWindow(hwndDocked); // deletes allocation too OnFinalMessage
      if(IsWindow(hwndFloated) ) 
			DestroyWindow(hwndFloated);
	}	
};

#define ATL_IDW_DOCKPANE_FIRST ATL_IDW_COMMAND_BAR+10

typedef CSimpleValArray<DOCKCONTEXT*> CDockMap;

// extra information during repositioning
struct DOCK_TRACKINFO 
{
   HWND hWnd;
   DOCKCONTEXT* pCtx;
	// all in screen coordinates
   POINT ptPos;   // current position
   POINT ptStart; // start pointer
   RECT rc;       // recommended dock rectangle
   short Side;
};

// information for default bar state 
struct DOCK_DEFBARSTATE
{
	UINT idDockPane;
	short Side;
	short bVisible;
};

///////////////////////////////////////////////////////
// CDockSplitterBar
// used for inbetween pane splitters and also for edge resizing

#pragma warning(disable : 4100)

// ~nab: no need to have per-template static instances
static HCURSOR s_hVertCursor = NULL, s_hHorizCursor = NULL;

// note this isn't a window
template< class T >
class CDockSplitterBar
{
public:
   LONG m_cxySplitter;
   bool m_bTracking;
   bool m_bDragging;
   CDCHandle m_dc;
	HWND m_hwReference;
	// all these rects etc are in screen coordinates
   POINT m_ptStartDragPoint;
   POINT m_ptEndDragPoint;
   POINT m_ptDeltaDragPoint;
   RECT  m_rcTracker; // ghost bar is drawn here
   SIZE  m_sizeTracker;
   RECT  m_rcTrackerBounds;

   CDockSplitterBar() :
      m_bTracking(false), m_bDragging(false)
   {
      if( s_hVertCursor == NULL ) {
         ::EnterCriticalSection(&_Module.m_csStaticDataInit);
         s_hVertCursor = ::LoadCursor(NULL, IDC_SIZENS);
         s_hHorizCursor = ::LoadCursor(NULL, IDC_SIZEWE);
         ::LeaveCriticalSection(&_Module.m_csStaticDataInit);
			ATLASSERT(s_hVertCursor);
      }
      m_sizeTracker.cx = ::GetSystemMetrics(SM_CXSIZEFRAME);
		if(m_sizeTracker.cx > 4)
			m_sizeTracker.cx = 4; // throttle for vista+
      m_sizeTracker.cy = ::GetSystemMetrics(SM_CYSIZEFRAME);
		if(m_sizeTracker.cy > 4)
			m_sizeTracker.cy = 4; // throttle for vista+
      m_cxySplitter = DEFAULT_SPLITTER_SIZE;
   }

   LRESULT OnSetCursor(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
   {
      T* pT = static_cast<T*>(this);
      DWORD dwPos = ::GetMessagePos();
      POINT ptPos = { GET_X_LPARAM(dwPos), GET_Y_LPARAM(dwPos) };
      pT->ScreenToClient(&ptPos);
      if( ::PtInRect(&pT->m_rcSplitter, ptPos) ) return 1;
      bHandled = FALSE;
      return 0;
   }

	// sliding of the splitter bar
   void DrawGhostBar()
   {
      ATLASSERT(!m_dc.IsNull());
      RECT rect = m_rcTracker;
      if( !::IsRectEmpty(&rect) ) {
			ATLASSERT(::GetDesktopWindow() != m_hwReference);
			// map rectangle to reference window coordinates
	      //::MapWindowPoints(0, m_hwReference, (LPPOINT)&rect, 2); mainframe client!
			RECT rcWin;
			::GetWindowRect(m_hwReference, &rcWin);
         ::OffsetRect(&rect, -rcWin.left, -rcWin.top);

         // Invert the brush pattern (looks just like frame window sizing)
         CBrush brush = CDCHandle::GetHalftoneBrush();
         if( brush.m_hBrush != NULL ) {
            CBrushHandle brushOld = m_dc.SelectBrush(brush);
            m_dc.PatBlt(rect.left, rect.top, rect.right - rect.left, rect.bottom - rect.top, PATINVERT);
            m_dc.SelectBrush(brushOld);
         }
      }
   }

	// here we draw the whole window perimeter when dragging
   void DrawDragBar()
   {
      ATLASSERT(!m_dc.IsNull());
      RECT rect = m_rcTracker;
      if( !::IsRectEmpty(&rect) ) {
			ATLASSERT(::GetDesktopWindow()==m_hwReference); // no mapping required

			// with themes this drawing is very slow, at least I don't do 4 separate blits!
			m_dc.DrawDragRect(&rect, m_sizeTracker, 0, m_sizeTracker);
#if 0
         // Invert the brush pattern (looks just like frame window sizing)
         CBrush brush = CDCHandle::GetHalftoneBrush();
         if( brush.m_hBrush != NULL ) {
            CBrushHandle brushOld = m_dc.SelectBrush(brush);
				// m_rcTracker is the whole window, we paint the edges just
            m_dc.PatBlt(rect.left + m_sizeTracker.cx, rect.top, rect.right - rect.left - (m_sizeTracker.cx * 2), m_sizeTracker.cy, PATINVERT);
            m_dc.PatBlt(rect.left, rect.bottom - m_sizeTracker.cy, rect.right - rect.left, m_sizeTracker.cy, PATINVERT);
            m_dc.PatBlt(rect.left, rect.top, m_sizeTracker.cx, rect.bottom - rect.top - m_sizeTracker.cy, PATINVERT);
            m_dc.PatBlt(rect.right - m_sizeTracker.cx, rect.top, m_sizeTracker.cx, rect.bottom - rect.top - m_sizeTracker.cy, PATINVERT);
            m_dc.SelectBrush(brushOld);
         }
#endif
      }
   }

	// splitter is plain and gets its borders from the titlebar and the clientedge of child windows
   void DrawSplitterBar(CDCHandle dc, bool bVertical, RECT& rect)
   {      
      if( ::IsRectEmpty(&rect) ) return;
      dc.FillRect(&rect, ::GetSysColorBrush(COLOR_3DFACE));
		// avoid the ugly VC6 style raised edge
      //dc.DrawEdge(&rect, EDGE_RAISED, (bVertical ? (BF_TOP|BF_BOTTOM) : (BF_LEFT|BF_RIGHT)));
   }

	// this is unnecessarily complicated but i can't be @rsed to change it
   bool PtInSplitter(POINT& pt, short Side, DWORD dwFlags, RECT& rcSplitter)
   {
      if( !IsDocked(Side) ) return false;
      if( m_bTracking ) return false;
      if( !::PtInRect(&rcSplitter, pt) ) return false;
      //if( (dwFlags & DCK_NOSPLITTER) != 0 ) return false;
      return true;
   }

   // Track loop

	// from what i see he has borrowed the MFC code (DOCKCONT.CPP)
   bool Track(bool bDragging)
   {
      T* pT = static_cast<T*>(this);
      StartTracking(bDragging);
      // Get messages until capture lost or cancelled/accepted
      while( ::GetCapture() == pT->m_hWnd ) {
         MSG msg;
			// as we don't have a windowproc we need a local pump
			// get messages for our OWN window, so that thread messages aren't lost
         if( !::GetMessage(&msg, pT->m_hWnd, 0, 0) ) {
            ::PostQuitMessage(msg.wParam);
            break;
         }
         switch( msg.message ) {
         case WM_LBUTTONUP:
            if( m_bDragging ) pT->OnEndDrag(); else pT->OnEndResize();
            CancelTracking();
            return true;
         case WM_MOUSEMOVE:
				// screen coordinates continue
            if( m_bDragging ) pT->OnMove(msg.pt); else pT->OnStretch(msg.pt);
            break;
         case WM_KEYUP:
            if( m_bDragging ) pT->OnKey((int) msg.wParam, false) ;
            break;
         case WM_KEYDOWN:
            if( m_bDragging ) pT->OnKey((int) msg.wParam, true);
            if( msg.wParam == VK_ESCAPE ) {
               CancelTracking();
               return false;
            }
            break;
         case WM_SYSKEYDOWN:
         case WM_LBUTTONDOWN:
         case WM_RBUTTONDOWN:
            CancelTracking();
            return false;      
         default:
            // Just dispatch rest of the messages
				ATLASSERT(msg.hwnd == pT->m_hWnd || ::IsChild(pT->m_hWnd, msg.hwnd) );
            ::DispatchMessage(&msg);
            break;
         }
      }
      CancelTracking();
      return false;
   }

protected:
   void StartTracking(bool bDragging)
   {
      ATLASSERT(!m_bTracking);
      T* pT = static_cast<T*>(this);
      // Capture window
      m_bTracking = true;
      pT->SetCapture();
      // Make sure no updates are pending
      pT->RedrawWindow(NULL, NULL, RDW_ALLCHILDREN|RDW_UPDATENOW);
      // Lock Window update while dragging over desktop
      ATLASSERT(m_dc.IsNull());
		
      HWND hWnd;
		if(bDragging) {
			// gettoplevelwindow isn't a good idea because we could be floating outside that!
			hWnd = ::GetDesktopWindow();
			BOOL bLocked = ::LockWindowUpdate(hWnd);
			m_dc = ::GetDCEx(hWnd, NULL, bLocked ? (DCX_WINDOW|DCX_CACHE|DCX_LOCKWINDOWUPDATE) : (DCX_WINDOW|DCX_CACHE));
		}
		else {
			// for tracking get lesser impact local window DC (vital for vista/7+)
			// for vista+ better avoid locking the desktop, really slow updates!
			hWnd = pT->GetTopLevelParent();
			m_dc = ::GetWindowDC(hWnd);
		}
		m_hwReference = hWnd;
		ATLASSERT(IsWindow(hWnd));
      ATLASSERT(!m_dc.IsNull());
      // Draw the initial focus rect
      m_bDragging = bDragging;
      if( m_bDragging ) DrawDragBar(); else DrawGhostBar();
      return;
   }

   void CancelTracking()
   {
      ATLASSERT(m_bTracking);
      if( !m_bTracking ) return;
      // Erase the focus rect
      if( m_bDragging ) {
			DrawDragBar(); 
			// Let window updates free
			::LockWindowUpdate(NULL);
		}
		else 
			DrawGhostBar();
      
      HWND hWnd = m_hwReference;
      if( !m_dc.IsNull() ) ::ReleaseDC(hWnd, m_dc.Detach());
      // Release the capture
      ::ReleaseCapture();
      m_bTracking = false;
   }

   // Overridables
public:
   void OnEndDrag() { };
   void OnEndResize() { };
   void OnKey(int nCode, bool bDown) { }; // unused
   void OnMove(POINT& pt) { };
   void OnStretch(POINT& pt) { };
};


#pragma warning(default : 4100)


///////////////////////////////////////////////////////
// CFloatingWindow
// this window is top level and parentless, careful with forwarding messages

// jump notifications straight to the top level parent to avoid losing time forwarding at each level
#define FORWARD_NOTIFICATIONS_WND(hw) \
	{ \
		bHandled = TRUE; \
		lResult = Atl3ForwardNotifications(hw, uMsg, wParam, lParam, bHandled); \
		if(bHandled) \
			return TRUE; \
	}

#if (_ATL_VER >= 0x0700)
// support the old one which allows for a window handle
static LRESULT Atl3ForwardNotifications(HWND hWnd, UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
{
	LRESULT lResult = 0;
	switch(uMsg)
	{
	case WM_COMMAND:
	case WM_NOTIFY:
#ifndef _WIN32_WCE
	case WM_PARENTNOTIFY:
#endif // !_WIN32_WCE
	case WM_DRAWITEM:
	case WM_MEASUREITEM:
	case WM_COMPAREITEM:
	case WM_DELETEITEM:
	case WM_VKEYTOITEM:
	case WM_CHARTOITEM:
	case WM_HSCROLL:
	case WM_VSCROLL:
	case WM_CTLCOLORBTN:
	case WM_CTLCOLORDLG:
	case WM_CTLCOLOREDIT:
	case WM_CTLCOLORLISTBOX:
	case WM_CTLCOLORMSGBOX:
	case WM_CTLCOLORSCROLLBAR:
	case WM_CTLCOLORSTATIC:
		lResult = ::SendMessage(::GetParent(hWnd), uMsg, wParam, lParam);
		break;
	default:
		bHandled = FALSE;
		break;
	}
	return lResult;
}
#endif

typedef CWinTraits<WS_OVERLAPPED|WS_CAPTION|WS_THICKFRAME|WS_SYSMENU, WS_EX_TOOLWINDOW|WS_EX_WINDOWEDGE> CFloatWinTraits;

template< class T, class TBase = CWindow, class TWinTraits = CFloatWinTraits >
class ATL_NO_VTABLE CFloatingWindowImpl : 
   public CWindowImpl< T, TBase, TWinTraits >,
   public CDockSplitterBar< T >
{
public:
   DECLARE_WND_CLASS_EX(NULL, CS_DBLCLKS, NULL)

   typedef CFloatingWindowImpl< T , TBase, TWinTraits > thisClass;
   
   BEGIN_MSG_MAP(thisClass)
      MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBackground)
      MESSAGE_HANDLER(WM_SIZE, OnSize)
      MESSAGE_HANDLER(WM_SYSCOMMAND, OnSysCommand)
      MESSAGE_HANDLER(WM_MENUCHAR, OnMenuChar)
      MESSAGE_HANDLER(WM_SETFOCUS, OnSetFocus)
      MESSAGE_HANDLER(WM_GETMINMAXINFO, OnMsgForward)
      MESSAGE_HANDLER(WM_DOCK_UPDATELAYOUT, OnSize)
      //MESSAGE_HANDLER(WM_NCACTIVATE, OnNcActivate)
      MESSAGE_HANDLER(WM_NCLBUTTONDOWN, OnLeftButtonDown)
      MESSAGE_HANDLER(WM_NCRBUTTONDOWN, OnRightButtonDown)
      MESSAGE_HANDLER(WM_NCLBUTTONDBLCLK, OnButtonDblClick)
#ifdef _DEBUG
		/* NOTIFICATION CIRCULATION
		 * As I discovered, client windows send notifications to their ORIGINAL parent
		 * (the one at the time of creation), and all the other calls to SetParent won't
		 * affect the routing of WM_NOTIFY. So if you create a listview as a child to the
		 * CMainFrame, then it will send there the WM_NOTIFY all the time. Which we like
		 * anyway as we don't want to fool around with forwarding messages up the templates!
		 * (perhaps CCM_SETNOTIFYWINDOW could be used to fully reparent a window)
		 */
		ATLASSERT(uMsg != WM_NOTIFY);
		ATLASSERT(m_pCtx->hwndRoot);
#endif
		FORWARD_NOTIFICATIONS_WND(m_pCtx->hwndRoot)
   END_MSG_MAP()

   DOCKCONTEXT* m_pCtx;

   CFloatingWindowImpl(DOCKCONTEXT* pCtx) :
      m_pCtx(pCtx)
   { 
   }

   HWND Create(HWND hWndParent, RECT& rcPos, LPCTSTR szWindowName = NULL,
         DWORD dwStyle = 0, DWORD dwExStyle = 0,
         UINT nID = 0, LPVOID lpCreateParam = NULL)
   {
      ATLASSERT(m_pCtx);
      if( (m_pCtx->dwFlags & DCK_NOHIDE) != 0 ) dwStyle = T::GetWndStyle(dwStyle) & ~WS_SYSMENU;
      return CWindowImpl< T, TBase, TWinTraits >::Create(hWndParent, rcPos, szWindowName, dwStyle, dwExStyle, nID, lpCreateParam);
   }

   virtual void OnFinalMessage(HWND /*hWnd*/)
   {
      delete (T*) this;
   }

   // Message handlers

   LRESULT OnEraseBackground(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      return 1; // handled, no background painting needed
   }

   LRESULT OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      T* pT = static_cast<T*>(this);
      pT->UpdateLayout();
      return 0;
   }

	// this allows an active titlebar all the time, but you must also do it for the mainframe
	// for an extremely complicated workaround see http://www.catch22.net/tuts/docking1
   LRESULT OnNcActivate(UINT uMsg, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
   {
      return DefWindowProc(uMsg, IsWindowEnabled(), lParam);
   }

   LRESULT OnLeftButtonDown(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& bHandled)
   {
      if( wParam == HTCAPTION ) {
			SetFocus();
         // Get cursor point and start a tracking look
         POINT pt = { GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) };
         m_ptStartDragPoint = m_ptEndDragPoint = pt;
         GetWindowRect(&m_rcTracker);
         // Enter tracking loop
         bool res = Track(true);
         if( res ) {
            // Determine if we landed over a docking pane or just moved around...
            DOCK_TRACKINFO ti = { m_hWnd, m_pCtx, m_ptEndDragPoint.x, m_ptEndDragPoint.y, m_ptStartDragPoint.x, m_ptStartDragPoint.y };
            ::SendMessage(m_pCtx->hwndRoot, WM_DOCK_QUERYTRACK, 0, (LPARAM) &ti);
            if( IsFloating(ti.Side) ) {
               MoveWindow(&ti.rc, TRUE);
			      m_pCtx->rcWindow = ti.rc;
            }
            else {
               ::SendMessage(m_pCtx->hwndRoot, WM_DOCK_UNFLOAT, 0, (LPARAM) m_pCtx);
               ::SendMessage(m_pCtx->hwndRoot, WM_DOCK_DOCK, ti.Side, (LPARAM) &ti);
            }
            return 0;
         }
         return 1;
      }

		// default processing for normal window sizing
      bHandled = FALSE;
      return 0;
   }

   LRESULT OnRightButtonDown(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
   {
      // We don't particular like the right - mousebutton. We need to
      // disable the system menu because it causes problems...
      if( wParam == HTCAPTION ) return 1;
      bHandled = FALSE;
      return 0;
   }

   LRESULT OnButtonDblClick(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
   {
      // Don't like double-clicks either. We need to disable
      // maximize on double-click for floating windows as well.
      if( wParam == HTCAPTION ) return 1;
      bHandled = FALSE;
      return 0;
   }

   LRESULT OnMenuChar(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      // Forward a failed menu-key strike to main window.
      // Perhaps it will show the menu for this key.
      LRESULT lRes = DefWindowProc(uMsg, wParam, lParam);
      if( HIWORD(lRes) == MNC_IGNORE ) {
         CWindow wndRoot = m_pCtx->hwndRoot;
         return ::SendMessage(wndRoot.GetTopLevelParent(), uMsg, wParam, lParam);
      }
      return lRes;
   }

   LRESULT OnSetFocus(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      LRESULT lRes = DefWindowProc(uMsg, wParam, lParam);
      ::SetFocus(m_pCtx->hwndChild);
      return lRes;
   }

   LRESULT OnSysCommand(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& bHandled)
   {
      switch( wParam & 0xFFF0 ) {
      case SC_CLOSE:
         if( (m_pCtx->dwFlags & DCK_NOHIDE) != 0 ) return 0;
         // Use post message so it comes async
         ::PostMessage(m_pCtx->hwndRoot, WM_DOCK_CLIENT_CLOSE, WM_DOCK_UNFLOAT, (LPARAM) m_pCtx);
         return 0;
      }
      bHandled = FALSE;
      return 0;
   }

   LRESULT OnMsgForward(UINT uMsg, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      // Forward to view
      return ::SendMessage(m_pCtx->hwndChild, uMsg, wParam, lParam);
   }

   // Track methods

   void OnMove(POINT& pt)
   {
      DOCK_TRACKINFO ti = { m_hWnd, m_pCtx, pt.x, pt.y, m_ptStartDragPoint.x, m_ptStartDragPoint.y };
      ::SendMessage(m_pCtx->hwndRoot, WM_DOCK_QUERYTRACK, 0, (LPARAM) &ti);
      if( !::EqualRect(&m_rcTracker, &ti.rc) ) {
         DrawDragBar();
         m_rcTracker = ti.rc;
         DrawDragBar();
      }
      m_ptEndDragPoint = pt;
   }

   // Overridables

   void UpdateLayout()
   {
      RECT rc;
      GetWindowRect(&rc);
      //m_pCtx->sizeFloat.cx = rc.right - rc.left;
      //m_pCtx->sizeFloat.cy = rc.bottom - rc.top;
      m_pCtx->rcWindow = rc;
      GetClientRect(&rc);
      ::SetWindowPos(m_pCtx->hwndChild, NULL, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, SWP_NOZORDER);
   }
};

class CFloatingWindow : public CFloatingWindowImpl<CFloatingWindow>
{
public:
   DECLARE_WND_CLASS_EX(_T("WTL_FloatingWindow"), CS_DBLCLKS, NULL)

   CFloatingWindow(DOCKCONTEXT* ctx) :
      CFloatingWindowImpl<CFloatingWindow>(ctx)
   { 
   }
};


///////////////////////////////////////////////////////
// CDockingPaneChildWindow
// host for individual docked child views

template< class T, class TBase = CWindow, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CDockingPaneChildWindowImpl : 
   public CWindowImpl< T, TBase, TWinTraits >,
   public CDockSplitterBar< T >
{
public:
   DECLARE_WND_CLASS_EX(NULL, CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, NULL)

   typedef CDockingPaneChildWindowImpl< T , TBase, TWinTraits > thisClass;
   
   BEGIN_MSG_MAP(thisClass)
      MESSAGE_HANDLER(WM_PAINT, OnPaint)
      MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBackground)
      MESSAGE_HANDLER(WM_SETCURSOR, OnSetCursor)
      MESSAGE_HANDLER(WM_MOUSEMOVE, OnMouseMove)
		MESSAGE_HANDLER(WM_MOUSELEAVE, OnMouseLeave);
      MESSAGE_HANDLER(WM_LBUTTONDOWN, OnButtonDown)
      MESSAGE_HANDLER(WM_LBUTTONUP, OnButtonUp)
      MESSAGE_HANDLER(WM_SIZE, OnSize)
      MESSAGE_HANDLER(WM_DOCK_UPDATELAYOUT, OnSize)
      MESSAGE_HANDLER(WM_DOCK_SETSPLITTER, OnSetSplitter)
#ifdef _DEBUG
		ATLASSERT(uMsg != WM_NOTIFY);
		ATLASSERT(m_pCtx->hwndRoot);
#endif
		// i don't think there are any notifications to forward but hey!
		FORWARD_NOTIFICATIONS_WND(m_pCtx->hwndRoot)
   END_MSG_MAP()

   DOCKCONTEXT* m_pCtx;
	// all following rects in client coordinates
   RECT m_rcChild;
   RECT m_rcSplitter;
   RECT m_rcGripper;
   RECT m_rcCloseButton;
   int m_cxyGripper;
   int m_cxyCloseButton;
   bool m_fCloseCapture; // now through _TrackMouseEvent
   bool m_fCloseDown, m_bCloseHot;
	CFont m_fCaption, m_f90deg, m_fSmall;

   CDockingPaneChildWindowImpl(DOCKCONTEXT* pCtx) :
      m_pCtx(pCtx), 
      m_fCloseDown(false), 
		m_bCloseHot(false),
      m_fCloseCapture(false)
   {
		// Caption font
		NONCLIENTMETRICS ncm = { 0 };
		ncm.cbSize = sizeof(ncm);
		::SystemParametersInfo(SPI_GETNONCLIENTMETRICS, sizeof(ncm), &ncm, 0);
		LOGFONT& lf = ncm.lfMenuFont; //ncm.lfSmCaptionFont;
		lf.lfWeight = FW_NORMAL; // vertical doesn't look good bold
		m_fCaption.CreateFontIndirect(&lf);

		// get a rotated version for vertical text
		// for win9x small caption font doesn't work vertically
		lf.lfEscapement = 900;                  // Rotate 90 degrees
		lf.lfClipPrecision |= CLIP_LH_ANGLES;   // For Win95 compatibility (Q82932)
		lf.lfOutPrecision = OUT_TT_ONLY_PRECIS; // Choose a truetype font (Q72021)
		m_f90deg.CreateFontIndirect(&lf);
		ATLASSERT(m_f90deg.m_hFont);

		// finally a font for the X close button
		// for real dpi i might need SetProcessDPIAware? or through manifest
		HDC screen = ::GetDC(0); 
		int ppi = ::GetDeviceCaps(screen, LOGPIXELSX);
		::ReleaseDC(0, screen); 

	   int pointsize = MulDiv(60, 96, ppi); // 6 points at 96 ppi
		m_fSmall.CreatePointFont(pointsize, _T("Marlett")); // this is a symbol font
		ATLASSERT(m_fSmall.m_hFont); // exists everywhere?

      ::SetRectEmpty(&m_rcChild);
      ::SetRectEmpty(&m_rcGripper);
      ::SetRectEmpty(&m_rcSplitter);
      ::SetRectEmpty(&m_rcCloseButton);
      //m_cxyGripper = ::GetSystemMetrics(SM_CYCAPTION)*4/5;
      //m_cxyCloseButton = m_cxyGripper-2;
      m_cxyGripper = ::GetSystemMetrics(SM_CYSMCAPTION);
		LOGFONT lfsm;
#ifdef _DEBUG
		m_fCaption.GetLogFont(&lfsm);
		ATLASSERT(abs(lfsm.lfHeight) < m_cxyGripper);
#endif

      //m_cxyCloseButton = ::GetSystemMetrics(SM_CYSMSIZE)-4; // too big when themes are on
		// use the font to determine size
		m_fSmall.GetLogFont(&lfsm);
		m_cxyCloseButton = abs(lfsm.lfHeight) + 3;
   }

protected:
   virtual void OnFinalMessage(HWND /*hWnd*/)
   {
      delete (T*) this;
   }

   LRESULT OnEraseBackground(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      return 1; // handled, no background painting needed
   }

   LRESULT OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      T* pT = static_cast<T*>(this);
      pT->UpdateLayout();
      return 0;
   }

	// everything is painted as client
   LRESULT OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      T* pT = static_cast<T*>(this);
      CPaintDC dc(m_hWnd);
      dc.ExcludeClipRect(&m_rcChild);
      RECT rc;
      GetClientRect(&rc);
      dc.FillRect(&rc, ::GetSysColorBrush(COLOR_3DFACE));
      short Side = m_pCtx->Side;
		ATLASSERT(IsDocked(Side));
      bool bVertical = IsDockedVertically(Side);
      if( m_cxySplitter > 0 ) pT->DrawSplitterBar((HDC) dc, bVertical, m_rcSplitter);
      pT->DrawGripperBar((HDC) dc, Side, m_rcGripper);
      pT->DrawPaneFrame((HDC) dc, Side, rc);
      return 0;
   }

   LRESULT OnMouseMove(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& bHandled)
   {
      POINT ptPos = { GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) };

		bool oldhot = m_bCloseHot;
		m_bCloseHot = ::PtInRect(&m_rcCloseButton, ptPos) != NULL;
      if( m_bCloseHot ) {
			if (!m_fCloseCapture)
			{
				m_fCloseCapture = TRUE;
				TRACKMOUSEEVENT tme;
				tme .cbSize = sizeof (tme);
				tme .dwFlags = TME_LEAVE;
				tme .hwndTrack = m_hWnd;
				_TrackMouseEvent (&tme); // tracks the whole window
			}
      }
      else if( PtInSplitter(ptPos, m_pCtx->Side, m_pCtx->dwFlags, m_rcSplitter) ) {
         bool bVertical = IsDockedVertically(m_pCtx->Side);
         ::SetCursor( bVertical ? s_hVertCursor : s_hHorizCursor);
         bHandled = FALSE;
      }

		if(oldhot != m_bCloseHot) {
			if(!m_bCloseHot)
				m_fCloseDown = 0;	
			InvalidateRect(&m_rcCloseButton);
		}
      return 0;
   }

	LRESULT OnMouseLeave (UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL &bHandled) 
	{
		//ATLASSERT(m_fCloseCapture); sometimes after click
		m_fCloseCapture = false;
		if(m_bCloseHot || m_fCloseDown) {
			m_fCloseDown = false;
			m_bCloseHot = false;
			InvalidateRect(&m_rcCloseButton);
		}

		bHandled = FALSE;
		return FALSE;
	}

   LRESULT OnButtonDown(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
   {
      POINT ptPos = { GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) };
      if( m_bCloseHot ) {
         // Clicked on close button? Start a mouse capture look because you
         // can still move the mouse away from the button...
			ATLASSERT(m_fCloseCapture);
			ATLASSERT(!m_fCloseDown);
         m_fCloseDown = true;
         InvalidateRect(&m_rcCloseButton);
      }
      else if( PtInSplitter(ptPos, m_pCtx->Side, m_pCtx->dwFlags, m_rcSplitter) ) {
         // Clicked on splitter. Start dragging then...
         ::ClientToScreen(m_hWnd,& ptPos);
         m_ptStartDragPoint = m_ptEndDragPoint = m_ptDeltaDragPoint = ptPos;
         //
         RECT rcWin;
         GetWindowRect(&rcWin);
         m_rcTracker = m_rcSplitter;
         ::OffsetRect(&m_rcTracker, rcWin.left, rcWin.top);
         //
         ::GetWindowRect(::GetParent(m_hWnd),& m_rcTrackerBounds);
         ::InflateRect(&m_rcTrackerBounds, -MIN_DOCKPANE_SIZE, -MIN_DOCKPANE_SIZE);
         // Enter tracking loop
         bool res = Track(false);
         if( res ) {
            if( IsDockedVertically(m_pCtx->Side) ) {
               m_pCtx->rcWindow.bottom += m_ptEndDragPoint.y - m_ptStartDragPoint.y;
            }
            else {
               m_pCtx->rcWindow.right += m_ptEndDragPoint.x - m_ptStartDragPoint.x;
            }
            //m_pCtx->bKeepSize = true; must keep sizes of all earlier panes
            ::SendMessage(GetParent(), WM_DOCK_RESIZEPANE, 0, (LPARAM)m_pCtx);
         }
      }
      else if( ::PtInRect(&m_rcGripper, ptPos) ) {
         // Clicked on gripper? Allow user to move/drag the window out...
         ::ClientToScreen(m_hWnd,& ptPos);
         m_ptStartDragPoint = m_ptEndDragPoint = m_ptDeltaDragPoint = ptPos;
         GetWindowRect(&m_rcTracker);
         // Enter tracking loop
         bool res = Track(true);
         if( res ) {
            // Done dragging! Let's determine where the window went...
            DOCK_TRACKINFO ti = { m_hWnd, m_pCtx, m_ptEndDragPoint.x, m_ptEndDragPoint.y, m_ptStartDragPoint.x, m_ptStartDragPoint.y };
            ::SendMessage(m_pCtx->hwndRoot, WM_DOCK_QUERYTRACK, 0, (LPARAM)&ti);
            if( ti.Side == m_pCtx->Side ) {
               // Dragged window to same side. Move the window to top position...NOT
               //RECT rc;
               //GetWindowRect(&rc);
               //if( !::PtInRect(&rc, m_ptEndDragPoint) ) 
						::SendMessage(m_pCtx->hwndRoot, WM_DOCK_REPOSITIONWINDOW, ti.Side, (LPARAM) &ti);
            }
            else {
               // Move the window out in the open or to another pane...
               ::SendMessage(m_pCtx->hwndRoot, WM_DOCK_UNDOCK, 0, (LPARAM) m_pCtx);
               if( IsFloating(ti.Side) ) m_pCtx->rcWindow = ti.rc;
               ::SendMessage(m_pCtx->hwndRoot, IsFloating(ti.Side) ? WM_DOCK_FLOAT : WM_DOCK_DOCK, ti.Side, (LPARAM) &ti);
            }
         }
      }
      return 0;
   }

   LRESULT OnButtonUp(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
   {
      if( m_fCloseDown ) {
			ATLASSERT(m_fCloseCapture && m_bCloseHot);
         // We're in a mouse capture loop. That means we're trying to figure
         // out if the close button is being hit...
         m_fCloseCapture = m_fCloseDown = false;
			m_bCloseHot = false;
         InvalidateRect(&m_rcCloseButton);
#ifdef _DEBUG
         POINT ptPos = { GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) };
         ATLASSERT( ::PtInRect(&m_rcCloseButton, ptPos) );
#endif
         ::PostMessage(m_pCtx->hwndRoot, WM_DOCK_CLIENT_CLOSE, WM_DOCK_UNDOCK, (LPARAM) m_pCtx);
      }
      return 0;
   }

   // Track methods
public:
   void OnMove(POINT& pt) // called from splitter
   {
      DOCK_TRACKINFO ti = { m_hWnd, m_pCtx, pt.x, pt.y, m_ptStartDragPoint.x, m_ptStartDragPoint.y };
      ::SendMessage(m_pCtx->hwndRoot, WM_DOCK_QUERYTRACK, 0, (LPARAM)&ti);
      if( !::EqualRect(&m_rcTracker, &ti.rc) ) {
         DrawDragBar();
         m_rcTracker = ti.rc;
         DrawDragBar();
      }
      m_ptDeltaDragPoint = pt;
   }

   void OnStretch(POINT& pt)
   {
      DrawGhostBar();
      if( IsDockedVertically(m_pCtx->Side) ) {
         int nOffset = pt.y - m_ptDeltaDragPoint.y;
         if( m_rcTracker.top + nOffset <= m_rcTrackerBounds.top ) nOffset = m_rcTrackerBounds.top - m_rcTracker.top;
         if( m_rcTracker.bottom + nOffset >= m_rcTrackerBounds.bottom ) nOffset = m_rcTrackerBounds.bottom - m_rcTracker.bottom;
         ::OffsetRect(&m_rcTracker, 0,nOffset);
         m_ptDeltaDragPoint.y += nOffset;
      }
      else {
         int nOffset = pt.x - m_ptDeltaDragPoint.x;
         if( m_rcTracker.left + nOffset <= m_rcTrackerBounds.left ) nOffset = m_rcTrackerBounds.left - m_rcTracker.left;
         if( m_rcTracker.right + nOffset >= m_rcTrackerBounds.right ) nOffset = m_rcTrackerBounds.right - m_rcTracker.right;
         ::OffsetRect(&m_rcTracker, nOffset, 0);
         m_ptDeltaDragPoint.x += nOffset;
      }
      DrawGhostBar();
   }

   void OnEndResize()
   {
      m_ptEndDragPoint = m_ptDeltaDragPoint;
   }

   void OnEndDrag()
   {
      m_ptEndDragPoint = m_ptDeltaDragPoint;
   }

   LRESULT OnSetSplitter(UINT /*uMsg*/, WPARAM wParam, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
		// DCK_NOSPLITTER policy apply, but watchout for CDockingPaneWindow too
		ATLASSERT(DEFAULT_SPLITTER_SIZE==m_cxySplitter || 0==m_cxySplitter);

      m_cxySplitter = wParam;
      return 0;
   }

   // Overridables
protected:
   void UpdateLayout()
   {
      // Reposition splitter and gripper bars
      RECT rect;
      GetClientRect(&rect);
		if(IsRectEmpty(&rect))
			return;
		ATLASSERT(IsDocked(m_pCtx->Side));

      const int CHILD_GAP = 0;
      bool bRealVertical = IsDockedVertically(m_pCtx->Side);
      bool bVertical = bRealVertical;

		// splitter located always according to true orientation
		if( bVertical ) {
         ::SetRect(&m_rcSplitter, rect.left, rect.bottom - m_cxySplitter, rect.right, rect.bottom);
			rect.bottom -= m_cxySplitter;
		}
		else {
			::SetRect(&m_rcSplitter, rect.right - m_cxySplitter, rect.top, rect.right, rect.bottom);
			rect.right -= m_cxySplitter;
		}

		if(m_pCtx->dwFlags & DCK_MANUALSIDE)
			bVertical = !(m_pCtx->dwFlags & DCK_LEFTBAR);
		// else automatic orientation of titlebar
		else if(bVertical)
			m_pCtx->dwFlags &= ~DCK_LEFTBAR;
		else
			m_pCtx->dwFlags |= DCK_LEFTBAR;

		// figure out all the edges (etched)
#define EDGECXY 2
		m_pCtx->dwFlags &= ~DCK_ALLEDGE;

		if((m_pCtx->dwFlags & DCK_NOGRIPPER)==0) {
			// wouldn't it be easier to get all the edges in non-client area? (WM_NCCALCSIZE)
			bool swaps = bVertical != bRealVertical;
			if(bVertical || bRealVertical || DOCK_TOP==m_pCtx->Side) {
				m_pCtx->dwFlags |= DCK_EDGETOP;
				rect.top += EDGECXY;
				if( (DOCK_TOP==m_pCtx->Side && !swaps) || (DOCK_TOP!=m_pCtx->Side && swaps) )
					rect.top += EDGECXY;
			}
			if(0==bVertical || 0==bRealVertical) {
				m_pCtx->dwFlags |= DCK_EDGELEFT;
				rect.left += EDGECXY;
			}
			if(DOCK_BOTTOM==m_pCtx->Side || (bRealVertical && !m_cxySplitter) ) {
				m_pCtx->dwFlags |= DCK_EDGEBOTTOM;
				rect.bottom -= 2*EDGECXY;
			}

			int nGap = 0;
			if( bVertical ) {
				if( (m_pCtx->dwFlags & DCK_NOHIDE) == 0 ) {
					nGap = CHILD_GAP + m_cxyCloseButton;
					m_rcCloseButton.left = rect.right-nGap;
					m_rcCloseButton.top = rect.top + /*2 +*/ (m_cxyGripper - m_cxyCloseButton)/2;
					m_rcCloseButton.right = m_rcCloseButton.left + m_cxyCloseButton;
					m_rcCloseButton.bottom = m_rcCloseButton.top + m_cxyCloseButton;
				}
				::SetRect(&m_rcGripper, rect.left, rect.top, rect.right-nGap, rect.top + m_cxyGripper);
				rect.top += m_cxyGripper;
			}
			else {
				if( (m_pCtx->dwFlags & DCK_NOHIDE) == 0 ) {
					nGap = CHILD_GAP + m_cxyCloseButton;
					m_rcCloseButton.left = rect.left + (m_cxyGripper - m_cxyCloseButton)/2;
					m_rcCloseButton.top = rect.top /*+ 2*/ + (m_cxyGripper - m_cxyCloseButton)/2;
					m_rcCloseButton.right = m_rcCloseButton.left + m_cxyCloseButton;
					m_rcCloseButton.bottom = m_rcCloseButton.top + m_cxyCloseButton;
				}
				::SetRect(&m_rcGripper, rect.left, rect.top+nGap, rect.left + m_cxyGripper, rect.bottom);
				rect.left += m_cxyGripper;
			}
		}

      // Calculate the client window area
      ::InflateRect(&rect, -CHILD_GAP, -CHILD_GAP);
      m_rcChild = rect;

      ::SetWindowPos(m_pCtx->hwndChild, NULL, m_rcChild.left, m_rcChild.top, m_rcChild.right - m_rcChild.left, m_rcChild.bottom - m_rcChild.top, SWP_NOZORDER | SWP_NOACTIVATE);
   }

   void DrawPaneFrame(CDCHandle dc, short /*Side*/, const RECT& rc)
   {
		//bool bVertical = !(m_pCtx->dwFlags & DCK_LEFTBAR);
		//dc.DrawEdge((RECT*)&rc, EDGE_ETCHED, bVertical ? BF_TOP : BF_LEFT);

		UINT edge = m_pCtx->dwFlags & DCK_ALLEDGE;
		if(!edge)
			return;

		edge >>= 4;
		// BF_LEFT etc match exactly the DCK_EDGELEFT etc
		ATLASSERT(BF_LEFT == (DCK_EDGELEFT >> 4));
		ATLASSERT(BF_TOP == (DCK_EDGETOP >> 4));
		ATLASSERT(BF_RIGHT == (DCK_EDGERIGHT >> 4));
		dc.DrawEdge((RECT*)&rc, EDGE_ETCHED, edge);
   }

	// gradient defines (if not already defined)
#ifndef COLOR_GRADIENTACTIVECAPTION
#define COLOR_GRADIENTACTIVECAPTION     27
#define COLOR_GRADIENTINACTIVECAPTION   28
#define SPI_GETGRADIENTCAPTIONS         0x1008
#endif

   void DrawGripperBar(CDCHandle dc, short /*Side*/, const RECT& rcBar)
   {
      if( ::IsRectEmpty(&rcBar) ) return;

		// smarter paint from http://www.datamekanix.com/
		COLORREF crhi = ::GetSysColor(COLOR_BTNHIGHLIGHT), crsh = ::GetSysColor(COLOR_BTNSHADOW);
		bool bVertical = !(m_pCtx->dwFlags & DCK_LEFTBAR); //IsDockedVertically(Side);
		const int INSET = 4;
		RECT rcLine;

		if(m_pCtx->dwFlags & DCK_TITLETEXT) {
			rcLine = rcBar;
			if(bVertical)
				InflateRect(&rcLine, 0/*-INSET*/, -1);
			else 
				InflateRect(&rcLine, -1, 0/*-INSET*/);

			COLORREF clrText;
			if(m_pCtx->dwFlags & DCK_GRADIENTEXT) {
				BOOL bActive = ::IsChild(m_hWnd, ::GetFocus());
				COLORREF clrCptn = GetSysColor(bActive ? COLOR_ACTIVECAPTION : COLOR_INACTIVECAPTION);
				clrText = ::GetSysColor(bActive ? COLOR_CAPTIONTEXT : COLOR_INACTIVECAPTIONTEXT);

				// do a gradient fill if available
				BOOL bGradient = FALSE;
				::SystemParametersInfo(SPI_GETGRADIENTCAPTIONS, 0, &bGradient, 0);
				if (bGradient) {
					// gradient from left to right or from bottom to top
					// get second gradient color (the right end)
					COLORREF clrCptnRight = //m_bActive ? ::GetSysColor(COLOR_GRADIENTACTIVECAPTION) :
						::GetSysColor(/*COLOR_GRADIENTINACTIVECAPTION*/COLOR_3DFACE); // @@@ this could lead to invisible text
						
					COLORREF one = bVertical ? clrCptn : clrCptnRight;
					TRIVERTEX        vert[2] ;
					GRADIENT_RECT    gRect;
					vert [0] .x      = rcLine.left;
					vert [0] .y      = rcLine.top;
					vert [0] .Red    = GetRValue(one) << 8;
					vert [0] .Green  = GetGValue(one) << 8;
					vert [0] .Blue   = GetBValue(one) << 8;
					vert [0] .Alpha  = 0x0000;
					
					one = bVertical ? clrCptnRight : clrCptn;
					vert [1] .x      = rcLine.right;
					vert [1] .y      = rcLine.bottom; 
					vert [1] .Red    = GetRValue(one) << 8;;
					vert [1] .Green  = GetGValue(one) << 8;
					vert [1] .Blue   = GetBValue(one) << 8;
					vert [1] .Alpha  = 0x0000;

					gRect.UpperLeft  = 0;
					gRect.LowerRight = 1;
					// @@@ problemo for win95? (win98 ok)
					GradientFill(dc,vert,2,&gRect,1, bVertical ? GRADIENT_FILL_RECT_H : GRADIENT_FILL_RECT_V);
				}
				else 
					dc.FillSolidRect(&rcLine, clrCptn); // solid color
			}
			else
				// background COLOR_3DFACE OnPaint
				clrText = ::GetSysColor(COLOR_BTNTEXT);

			// get the window text
			TCHAR szCaption[128] = { 0 };
			ATLASSERT(GetWindow(GW_CHILD)==m_pCtx->hwndChild);
			::GetWindowText(m_pCtx->hwndChild, szCaption, (sizeof(szCaption)/sizeof(TCHAR))-1);
			ATLASSERT(szCaption[0]);
			if(szCaption[0]) {
				dc.SetBkMode(TRANSPARENT);
				dc.SetTextColor(clrText);
				HFONT hOldFont = dc.SelectFont( bVertical ? (HFONT)m_fCaption : (HFONT)m_f90deg );
				if( bVertical ) {
					rcLine.left += 2;
					dc.DrawText(szCaption, -1, &rcLine, DT_SINGLELINE | DT_NOPREFIX | DT_END_ELLIPSIS | DT_VCENTER);
				}
				else {
					// @@@ this isn't properly centered in win9x
					::ExtTextOut(dc, rcLine.left-1, rcLine.bottom-3, ETO_CLIPPED, &rcLine, szCaption, ::lstrlen(szCaption), NULL);
				}
				dc.SelectFont(hOldFont);
			}
		}
		else {
			const int LINE_GAP = 3;
			if( bVertical ) {
				// each line is 3 pixels thick
				::SetRect(&rcLine, rcBar.left+INSET, rcBar.top+4, rcBar.right-INSET, rcBar.top+7);
			}
			else {
				::SetRect(&rcLine, rcBar.left+4, rcBar.top+INSET, rcBar.left+7, rcBar.bottom-INSET);
			}

			dc.Draw3dRect(&rcLine, crhi, crsh);
			::OffsetRect(&rcLine, bVertical ? 0 : LINE_GAP, bVertical ? LINE_GAP : 0);
			dc.Draw3dRect(&rcLine, crhi, crsh);
		}

		// ~nab: no way to draw a flat button with DrawFrameControl even with DFCS_FLAT
		// so I use a trick I saw in an MFC project
      if( !::IsRectEmpty(&m_rcCloseButton) ) {
         //dc.DrawFrameControl(&m_rcCloseButton, DFC_CAPTION, m_fCloseDown ? DFCS_FLAT|DFCS_CAPTIONCLOSE|DFCS_PUSHED : DFCS_FLAT |DFCS_CAPTIONCLOSE);

			if (m_fCloseDown)
				dc.Draw3dRect(&m_rcCloseButton, crsh, crhi);
			else if (m_bCloseHot)
				dc.Draw3dRect(&m_rcCloseButton, crhi, crsh);

			// the same thing shows even if we have themes enabled
			dc.SetBkMode(TRANSPARENT);
			dc.SetTextColor(::GetSysColor(COLOR_BTNTEXT));
			HFONT hOldFont = dc.SelectFont(m_fSmall);
			dc.TextOut(m_rcCloseButton.left + 2, m_rcCloseButton.top + 2, _T("r")); // x-like
			dc.SelectFont(hOldFont);
      }
   }
};

class CDockingPaneChildWindow : public CDockingPaneChildWindowImpl<CDockingPaneChildWindow>
{
public:
   DECLARE_WND_CLASS_EX(_T("WTL_DockingChildWindow"), CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, NULL)

   CDockingPaneChildWindow(DOCKCONTEXT* ctx) : 
      CDockingPaneChildWindowImpl<CDockingPaneChildWindow>(ctx) 
   { 
   }
};


///////////////////////////////////////////////////////
// CDockingPaneWindow
// one of these per dock side (4 altogether usually)

template< class T, class TBase = CWindow, class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CDockingPaneWindowImpl : 
   public CWindowImpl< T, TBase, TWinTraits >,
   public CDockSplitterBar< T >
{
public:
   DECLARE_WND_CLASS_EX(NULL, CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, COLOR_WINDOW)

   typedef CDockingPaneWindowImpl< T , TBase, TWinTraits > thisClass;
   
   BEGIN_MSG_MAP(thisClass)
      MESSAGE_HANDLER(WM_PAINT, OnPaint)
      MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBackground)
      MESSAGE_HANDLER(WM_SETCURSOR, OnSetCursor)
      MESSAGE_HANDLER(WM_MOUSEMOVE, OnMouseMove)
      MESSAGE_HANDLER(WM_LBUTTONDOWN, OnButtonDown)
      MESSAGE_HANDLER(WM_SIZE, OnSize)
      MESSAGE_HANDLER(WM_DOCK_UPDATELAYOUT, OnRecalcSpace)
      MESSAGE_HANDLER(WM_DOCK_RESIZEPANE, OnManageResize)

		ATLASSERT(uMsg != WM_NOTIFY);
		// no forwarding required as we skip straight to the frame one level down
   END_MSG_MAP()

   CDockMap m_map;       // Panels docking in this pane
                         // Pointers are owned by the parent window
   DWORD m_dwExtStyle;   // Mirror of extended style of parent window (dock window)
   int m_cy;             // Current size of panel
   int m_cyOld;          // Last size when panel was visible
   short m_Side;         // The side the panel is shown the
   RECT m_rcSplitter;    // Rectangle of the splitter

   CDockingPaneWindowImpl() : 
      m_cy(0), 
      m_cyOld(DEFAULT_DOCKPANE_SIZE), 
      m_dwExtStyle(0L)
   {
      ::SetRectEmpty(&m_rcSplitter);
   }

   // Implementation

   void DockWindow(DOCKCONTEXT* ctx, int iPos=-1/*default: add last*/)
   {
		ATLASSERT(m_hWnd);
      ATLASSERT(ctx);
		BOOL bPlaceHolder = ctx->rcWindow.bottom < 0 || ctx->rcWindow.right < 0; // hidden
		ATLASSERT(DOCK_HIDDEN==ctx->Side);
      if( CountVisiblePanes() == 0 && !bPlaceHolder ) {
         m_cy = m_cyOld;
         ShowWindow(SW_SHOWNOACTIVATE);
      }

		// is this an old hidden pane being reattached?
		BOOL hidden = m_map.Find(ctx) != -1;
		ATLASSERT(0==hidden || ctx->LastSide == m_Side);

		BOOL bFromFloat = IsFloating(ctx->LastSide);
      ctx->Side = bPlaceHolder ? DOCK_HIDDEN : m_Side;
      ctx->LastSide = m_Side;
      ::SetParent(ctx->hwndDocked, m_hWnd);
      ::SetParent(ctx->hwndChild, ctx->hwndDocked);

		if(!hidden) {
			m_map.Add(ctx);
			if(iPos >= 0) {
				int i, last = m_map.GetSize()-1;
				ATLASSERT(m_map[last]==ctx);
				if(last != iPos) {
					ATLASSERT(iPos < last);
					// move all panes up to make room for new one
					i = last;
					while(i > iPos) {
						DOCKCONTEXT* pdc = m_map[i-1];
						m_map.SetAtIndex(i, pdc);
						i--;
					}
					m_map.SetAtIndex(i, ctx);
				}
			}
		}

		::ShowWindow(ctx->hwndDocked, bPlaceHolder ? SW_HIDE : SW_SHOWNOACTIVATE);
		if(!bPlaceHolder) {
			PostMessage(WM_DOCK_UPDATELAYOUT);
			if(bFromFloat)
				// in some situations we don't get the sizing event if the dock size is the same
				// e.g. from dock->float, pane empty, then dock back to the same pane, boom!
				::PostMessage(ctx->hwndDocked, WM_SIZE, 0,0);
		}
		else
			SetRectEmpty(&ctx->rcWindow); // cancel those -1
   }

	// we don't get visual hilite during tracking but at least post facto dock correctly
	int DockPosition(DOCK_TRACKINFO* tri)
	{
		ATLASSERT(tri);
		ATLASSERT(m_hWnd);

		int i, nPanes = m_map.GetSize();
		if(!nPanes)
			return 0;
		if(!CountVisiblePanes())
			return 0;

		POINT pt = tri->ptPos;
      ::MapWindowPoints(0, m_hWnd, &pt, 1); // client

		for(i=0; i<nPanes; i++) {
         if( PtInRect(&m_map[i]->rcWindow, pt) ) {
				ATLASSERT(m_map[i]->Side == m_Side); // hidden sites have empty rcWindow
				// see if we are left or right from center of gravity
				LONG center;
				if(IsDockedVertically(m_Side)) {
					center = (m_map[i]->rcWindow.top + m_map[i]->rcWindow.bottom)/2;
					return i + (pt.y > center);
				}
				else {
					center = (m_map[i]->rcWindow.left + m_map[i]->rcWindow.right) / 2;
					return i + (pt.x > center);
				}
			}
		}

		ATLASSERT(0); // wrong exit
		return -1;
	}

   void UnDockWindow(DOCKCONTEXT* ctx, BOOL bRemove)
   {
		ATLASSERT(m_hWnd);
      ATLASSERT(ctx);
		ATLASSERT(ctx->Side == m_Side);
#ifdef _DEBUG
		RECT rc_;
		::GetWindowRect(ctx->hwndDocked, &rc_);
      ::MapWindowPoints(0, m_hWnd, (LPPOINT)&rc_, 2); // client
		ATLASSERT(EqualRect(&rc_, &ctx->rcWindow));
#endif

		int i = m_map.GetSize();
      while(i--) {
         if( m_map[i] == ctx ) {
				if(bRemove)
					m_map.RemoveAt(i);
				else
					SetRectEmpty(&ctx->rcWindow);
            ::ShowWindow(ctx->hwndDocked, SW_HIDE);
				ctx->Side = DOCK_HIDDEN; // must be set early
            break;
         }
      }
		ATLASSERT(i >= 0); // found?
      if( CountVisiblePanes() == 0 ) {
         if( (m_dwExtStyle & DCK_EX_REMEMBERSIZE) != 0 ) m_cyOld = m_cy; // remember pane size to until next time
         m_cy = 0;
         ShowWindow(SW_HIDE);
      }
      else {
         SendMessage(WM_DOCK_UPDATELAYOUT);
      }
   }

   void RepositionWindow(DOCKCONTEXT* ctx, int iPos)
   {
		ATLASSERT(m_hWnd);
		ATLASSERT(ctx->Side == m_Side);
		ATLASSERT(iPos >= 0);

		int i, nPanes = m_map.GetSize(); 
      if( iPos >=  nPanes)
			iPos = nPanes-1;
		else if(iPos < 0)
			iPos = 0;
      for( i = 0; i < nPanes; i++ ) {
         if( m_map[i] == ctx ) {
				if(i==iPos)
					return; // nop
            DOCKCONTEXT* pCtx = m_map[iPos];
            m_map.SetAtIndex(iPos, ctx);
            m_map.SetAtIndex(i, pCtx);
            break;
         }
      }
		ATLASSERT(i < nPanes);
      SendMessage(WM_DOCK_UPDATELAYOUT); // no posting

		// the child windows didn't receive any sizing events so manually update splitters etc
		for( i = 0; i < nPanes; i++ ) {
			if(m_map[i]->Side==DOCK_HIDDEN)
				continue;
         ::SendMessage(m_map[i]->hwndDocked, WM_DOCK_UPDATELAYOUT, 0, 0L);
      }

      //Invalidate();
   }

	// count all visible panes
	int CountVisiblePanes()
	{
		int i, nPanes = m_map.GetSize(), nVisible = 0;
      for( i = 0; i < nPanes; i++ ) {
			if(DOCK_HIDDEN==m_map[i]->Side) {
				ATLASSERT(!::IsWindowVisible(m_map[i]->hwndDocked));
				ATLASSERT(IsRectEmpty(&m_map[i]->rcWindow));
			}
			else {
				//ATLASSERT(::IsWindowVisible(m_map[i]->hwndDocked));
				nVisible++;
				ATLASSERT(m_Side==m_map[i]->Side);
			}
		}

		return nVisible;
	}

   // Message handlers
protected:
   LRESULT OnRecalcSpace(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
   {
		// peek and remove all WM_DOCK_UPDATELAYOUT
		// only dock events are posted to help us during initial bar state setup
		MSG msg;
		while(::PeekMessage(&msg, m_hWnd, WM_DOCK_UPDATELAYOUT, WM_DOCK_UPDATELAYOUT, PM_REMOVE));

      OnSize(0, 0, 0, bHandled);
      return 0;
   }

   LRESULT OnManageResize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
	{
		DOCKCONTEXT* ctx = (DOCKCONTEXT*)lParam;
		ATLASSERT(CountVisiblePanes());

		int i, nPanes = m_map.GetSize() -1 ; // one less
		for(i=0; i<nPanes; i++) {
			if(m_map[i]->Side != DOCK_HIDDEN)
				m_map[i]->bKeepSize = true;
			if(m_map[i] == ctx)
				break;
		}
		ATLASSERT(i<nPanes);
		SendMessage(WM_DOCK_UPDATELAYOUT);

		return 0;
	}
	

   LRESULT OnPaint(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      CPaintDC dc(m_hWnd);
      // Draw splitter along the pane side
      T* pT = static_cast<T*>(this);
      pT->DrawSplitterBar((HDC) dc, !IsDockedVertically(m_Side), m_rcSplitter);
      return 0;
   }

   LRESULT OnEraseBackground(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      return 1; // handled, no background painting needed
   }

   LRESULT OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
   {
      bHandled = FALSE;
		int visible = CountVisiblePanes();
      if( visible == 0 ) return 1;
      T* pT = static_cast<T*>(this);
      pT->UpdateLayout();
      HDWP hdwp = BeginDeferWindowPos(visible);
      for( int i = 0; i < m_map.GetSize(); i++ ) {
			if(m_map[i]->Side==DOCK_HIDDEN)
				continue;
         RECT& rc = m_map[i]->rcWindow;
         ::DeferWindowPos(hdwp, m_map[i]->hwndDocked, NULL, rc.left, rc.top, rc.right - rc.left, rc.bottom - rc.top, SWP_NOZORDER | SWP_NOACTIVATE);
      }
      EndDeferWindowPos(hdwp);
      return 1;
   }

	// ~nab: there is a similar handler in CDockingPaneChildWindowImpl but that's for inside the dock
   LRESULT OnMouseMove(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& bHandled)
   {
      POINT ptPos = { GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) };
      if( PtInSplitter(ptPos, m_Side, 0, m_rcSplitter) ) {
         bool bVertical = !IsDockedVertically(m_Side);
         ::SetCursor( bVertical ? s_hVertCursor : s_hHorizCursor);
         bHandled = FALSE;
      }
      return 0;
   }

   LRESULT OnButtonDown(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
   {
      POINT ptPos = { GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam) };
      if( PtInSplitter(ptPos, m_Side, 0, m_rcSplitter) ) {
         // Click on the spiltter? Determine how har we can drag the pane and
         // start tracking...
         ::ClientToScreen(m_hWnd,& ptPos);
         m_ptStartDragPoint = m_ptEndDragPoint = m_ptDeltaDragPoint = ptPos;
         //
         RECT rcWin;
         GetWindowRect(&rcWin);
         m_rcTracker = m_rcSplitter;
         ::OffsetRect(&m_rcTracker, rcWin.left, rcWin.top);
         //
         ::GetWindowRect(GetParent(),& m_rcTrackerBounds);
         RECT rcLimit;
         ::SendMessage(GetParent(), WM_DOCK_QUERYRECT, DOCK_INFO_CHILD, (LPARAM)&rcLimit);
         switch( m_Side ) {
         case DOCK_LEFT:   m_rcTrackerBounds.right = rcLimit.right; break;
         case DOCK_RIGHT:  m_rcTrackerBounds.left = rcLimit.left; break;
         case DOCK_TOP:    m_rcTrackerBounds.bottom = rcLimit.bottom; break;
         case DOCK_BOTTOM: m_rcTrackerBounds.top = rcLimit.top; break;
         }
         // The actual dragging area is slightly smaller than the client area
         ::InflateRect(&m_rcTrackerBounds, -MIN_DOCKPANE_SIZE, -MIN_DOCKPANE_SIZE);
         // Enter tracking loop
         bool res = Track(false);
         if( res ) {
            int nOffset = ( !IsDockedVertically(m_Side) ? (m_ptEndDragPoint.y - m_ptStartDragPoint.y) : (m_ptEndDragPoint.x - m_ptStartDragPoint.x) );
            if( m_Side == DOCK_RIGHT || m_Side == DOCK_BOTTOM ) nOffset = -nOffset;
            m_cy += nOffset;
            ::SendMessage(GetParent(), WM_DOCK_UPDATELAYOUT, 0, 0L);
				// changes will trickle down all the way from OnSize handlers
         }
      }
      return 0;
   }

	int LastVisibleIdx()
	{
		ATLASSERT(CountVisiblePanes());
		int i = m_map.GetSize();
		while(i--)
			if(DOCK_HIDDEN != m_map[i]->Side)
				return i;

		return -1;
	}

   // Track methods
public:
   void OnStretch(POINT& pt)
   {
      DrawGhostBar();
      if( !IsDockedVertically(m_Side) ) {
         int nOffset = pt.y - m_ptDeltaDragPoint.y;
         if( m_rcTracker.top + nOffset <= m_rcTrackerBounds.top ) nOffset = m_rcTrackerBounds.top - m_rcTracker.top;
         if( m_rcTracker.bottom + nOffset >= m_rcTrackerBounds.bottom ) nOffset = m_rcTrackerBounds.bottom - m_rcTracker.bottom;
         ::OffsetRect(&m_rcTracker, 0, nOffset);
         m_ptDeltaDragPoint.y += nOffset;
      }
      else {
         int nOffset = pt.x - m_ptDeltaDragPoint.x;
         if( m_rcTracker.left + nOffset <= m_rcTrackerBounds.left ) nOffset = m_rcTrackerBounds.left - m_rcTracker.left;
         if( m_rcTracker.right + nOffset >= m_rcTrackerBounds.right ) nOffset = m_rcTrackerBounds.right - m_rcTracker.right;
         ::OffsetRect(&m_rcTracker, nOffset, 0);
         m_ptDeltaDragPoint.x += nOffset;
      }
      DrawGhostBar();
   }

   void OnEndResize()
   {
      m_ptEndDragPoint = m_ptDeltaDragPoint;
   }

   // Overridables

   void UpdateLayout()
   {
		static BOOL s_bstrap_ = 0;
		ATLASSERT(m_hWnd);
		int nVisible = CountVisiblePanes();
      if( nVisible == 0 ) return;
      if( (GetStyle() & WS_VISIBLE) == 0 ) return;

      int nPanes = m_map.GetSize();
      bool bVertical = IsDockedVertically(m_Side);

      // Place side splitter for this docking area
      RECT rect; 
      GetClientRect(&rect);
		if(rect.bottom == rect.top)
			return; // must be while we are being created
      switch( m_Side ) {
      case DOCK_LEFT:
         ::SetRect(&m_rcSplitter, rect.right - m_cxySplitter, rect.top, rect.right, rect.bottom);
         rect.right -= m_cxySplitter;
         break;
      case DOCK_TOP:
         ::SetRect(&m_rcSplitter, rect.left, rect.bottom - m_cxySplitter, rect.right, rect.bottom);
         rect.bottom -= m_cxySplitter;
         break;
      case DOCK_RIGHT:
         ::SetRect(&m_rcSplitter, rect.left, rect.top, rect.left + m_cxySplitter, rect.bottom);
         rect.left += m_cxySplitter;
         break;
      case DOCK_BOTTOM:
         ::SetRect(&m_rcSplitter, rect.left, rect.top, rect.right, rect.top + m_cxySplitter);
         rect.top += m_cxySplitter;
         break;
      }

      // Place splitters in each child panel (except in the last one)
      int i, nLast = LastVisibleIdx();
      for( i = 0; i < nLast; i++ ) {
			if(m_map[i]->Side == DOCK_HIDDEN)
				continue;
         ::SendMessage(m_map[i]->hwndDocked, WM_DOCK_SETSPLITTER, DEFAULT_SPLITTER_SIZE, 0L);
      }
      // The last panel does not have a splitter
      ::SendMessage(m_map[nLast]->hwndDocked, WM_DOCK_SETSPLITTER, 0, 0L);

      // Get actual height of all panels
      int nActualHeight = 0;
      for( i = 0; i < nPanes; i++ ) {
			if(m_map[i]->Side == DOCK_HIDDEN)
				continue;
         const RECT& rc = m_map[i]->rcWindow;
         int iPaneHeight = (bVertical ? rc.bottom - rc.top : rc.right - rc.left);
         if( iPaneHeight < 10 ) iPaneHeight = MIN_DOCKPANE_SIZE;
         nActualHeight += iPaneHeight;
      }
      // Get height of docking area
      int nTop, nHeight;
      if( bVertical ) {
         nTop = rect.top;
         nHeight = rect.bottom - rect.top;
      }
      else {
         nTop = rect.left;
         nHeight = rect.right - rect.left;
      }
		ATLASSERT(!nTop); // so that nHeight is marking the real end
      // Distribute the difference among panels
      int nDelta = ((nHeight - nActualHeight) / nVisible);
		BOOL burst = 0;
      for( i = 0; i < nPanes; i++ ) {
			if(m_map[i]->Side == DOCK_HIDDEN)
				continue;
         const RECT& rc = m_map[i]->rcWindow;
         int nSize = (bVertical ? rc.bottom - rc.top : rc.right - rc.left);
         if( !m_map[i]->bKeepSize ) nSize += nDelta;
         if( nSize < MIN_DOCKPANE_SIZE ) nSize = MIN_DOCKPANE_SIZE;
         if( nTop + nSize >= nHeight && i < nLast ) {
				burst = 1; // must do this again
				nSize = MIN_DOCKPANE_SIZE;
			}
         if( bVertical ) {
            ::SetRect(&m_map[i]->rcWindow, rect.left, nTop, rect.right, nTop + nSize);
         }
         else {
            ::SetRect(&m_map[i]->rcWindow, nTop, rect.top, nTop + nSize, rect.bottom);
         }
         nTop += nSize;
         m_map[i]->bKeepSize = false;
      }

		if(burst) {
			ATLASSERT(nVisible > 1);
			ATLASSERT(!s_bstrap_); // successive failures? -> possible if we are extremely restricted in size
			if(!s_bstrap_) {
				nDelta = nHeight / nVisible;
				nTop = 0;
				for( i = 0; i < nPanes; i++ ) {
					if(m_map[i]->Side == DOCK_HIDDEN)
						continue;
					// make all panes equal
					RECT& rc = m_map[i]->rcWindow;
					if(bVertical) {
						rc.top = nTop;
						rc.bottom = nTop + nDelta;
					}
					else {
						rc.left = nTop;
						rc.right = nTop + nDelta;
					}

					nTop += nDelta;
				}
				s_bstrap_ = 1;
				UpdateLayout(); // try again with all the bKeepSize out
				s_bstrap_ = 0;
			}
		}
		else
			// Stretch the last window to the size of the docking window
			(bVertical ? m_map[nLast]->rcWindow.bottom : m_map[nLast]->rcWindow.right ) = nHeight;
   }
};

class CDockingPaneWindow : public CDockingPaneWindowImpl<CDockingPaneWindow>
{
public:
   DECLARE_WND_CLASS_EX(_T("WTL_DockingPaneWindow"), CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, COLOR_WINDOW)
};


///////////////////////////////////////////////////////
// CDockingWindow
// takes over the whole frame client area and manages the bizna

template< class T, 
          class TPaneWindow = CDockingPaneWindow,
          class TDockWindow = CDockingPaneChildWindow,
          class TFloatWindow = CFloatingWindow,
          class TBase = CWindow, 
          class TWinTraits = CControlWinTraits >
class ATL_NO_VTABLE CDockingWindowImpl : 
   public CWindowImpl< T, TBase, TWinTraits >
{
public:
   DECLARE_WND_CLASS_EX(NULL, CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, NULL)

   typedef CDockingWindowImpl< T, TPaneWindow, TDockWindow, TFloatWindow, TBase, TWinTraits > thisClass;
   
	CDockingWindowImpl() {
		m_idLastActive = -1; // pane ID
		m_dwExtStyle = 0;
		m_sizeLast.cx = 0;
		m_sizeLast.cy = 0;
		m_bNoUpdates = 0;
	}

   BEGIN_MSG_MAP(thisClass)
      MESSAGE_HANDLER(WM_CREATE, OnCreate)
      MESSAGE_HANDLER(WM_DESTROY, OnDestroy)
      MESSAGE_HANDLER(WM_ERASEBKGND, OnEraseBackground)
      MESSAGE_HANDLER(WM_SIZE, OnSize)
      MESSAGE_HANDLER(WM_DOCK_QUERYRECT, OnQueryRect)
      MESSAGE_HANDLER(WM_DOCK_QUERYTRACK, OnQueryTrack)
      MESSAGE_HANDLER(WM_DOCK_DOCK, OnDock)
      MESSAGE_HANDLER(WM_DOCK_FLOAT, OnFloat)
      MESSAGE_HANDLER(WM_DOCK_UNDOCK, OnUnDock)
      MESSAGE_HANDLER(WM_DOCK_UNFLOAT, OnUnFloat)
      MESSAGE_HANDLER(WM_DOCK_UPDATELAYOUT, OnSize)
      MESSAGE_HANDLER(WM_DOCK_REPOSITIONWINDOW, OnRepositionWindow)
      MESSAGE_HANDLER(WM_DOCK_CLIENT_CLOSE, OnClientClose)
		// rarely we could get notifications if you parent your windows at root dock
		FORWARD_NOTIFICATIONS()
   END_MSG_MAP()

protected:
   TPaneWindow m_panes[4];  // The 4 panel windows (one for each side) order like DOCK_xxx
   CDockMap m_map;          // The master map of dockable windows
   HWND m_hwndClient;       // The client window contained in the center
	UINT m_idLastActive;     // focus handling (active caption)
   DWORD m_dwExtStyle;      // Optional styles DCK_EX_xxx
   SIZE m_sizeBorder;       // Size of window borders
	SIZE m_sizeLast;         // size of last resize
	BOOL m_bNoUpdates;

   // Operations

public:
   void SetClient(HWND hWnd)
   {
      ATLASSERT(::IsWindow(hWnd));
      ATLASSERT(::GetWindowLong(hWnd, GWL_STYLE) & WS_CHILD);
      m_hwndClient = hWnd;
		// client isn't always our child, e.g. in MDI
   }

   DWORD GetExtendedDockStyle() const
   {
      return m_dwExtStyle;
   }

   void SetExtendedDockStyle(DWORD dwStyle)
   {
      ATLASSERT(!::IsWindow(m_hWnd)); // setup before creation!

      m_dwExtStyle = dwStyle;
      // Duplicate style settings to panels
      for( int i = 0; i < 4; i++ ) m_panes[i].m_dwExtStyle = dwStyle;
   }

	// if you want offset in array use _GetContext
   BOOL IsDockPane(HWND hWnd) const
   {
      ATLASSERT(::IsWindow(hWnd));
      for( int i = 0; i < m_map.GetSize(); i++ ) if( m_map[i]->hwndChild == hWnd ) return TRUE;      
      return FALSE;
   }

	// first add, then dock or float
   BOOL AddWindow(HWND hWnd, DWORD dwFlags = 0)
   {
      ATLASSERT(::IsWindow(hWnd));
      ATLASSERT(!IsDockPane(hWnd));

		UINT paneID = ::GetDlgCtrlID(hWnd);
		ATLASSERT(paneID >= ATL_IDW_DOCKPANE_FIRST && paneID < ATL_IDW_PANE_FIRST); // around 240 to play with
#ifdef _DEBUG
		// unique IDs are essential for registry persistence of settings
		for(int i=0; i<m_map.GetSize(); i++)
			ATLASSERT(m_map[i]->idDockPane != paneID);
#endif

      // Create docking context
      DOCKCONTEXT* ctx = new DOCKCONTEXT;
      if( ctx == NULL ) return FALSE;
      ctx->Side = DOCK_HIDDEN;
      ctx->LastSide = DOCK_HIDDEN; // that's how you tell something isn't docked/floated yet
      ctx->hwndChild = hWnd;
      ctx->hwndRoot = m_hWnd;
      ::SetRect(&ctx->rcWindow, 0, 0, MIN_DOCKPANE_SIZE, MIN_DOCKPANE_SIZE);
      //ctx->sizeFloat.cx = ctx->sizeFloat.cy = DEFAULT_FLOAT_SIZE;
      ctx->dwFlags = dwFlags;
      ctx->bKeepSize = false;
      ctx->hwndOrigPrnt = ::GetParent(hWnd);
		ctx->idDockPane = paneID;

		TFloatWindow* wndFloat = NULL;
      // Create docking child
      TDockWindow* wndDock = new TDockWindow(ctx);
		if(wndDock) {
			wndDock->Create(m_hWnd, rcDefault, NULL);
			ATLASSERT(::IsWindow(wndDock->m_hWnd));
			ctx->hwndDocked = *wndDock;

			// Create floating child
			TCHAR szCaption[128] = { 0 };    // max text length is 127 for floating caption
			::GetWindowText(hWnd, szCaption, (sizeof(szCaption) / sizeof(TCHAR))-1);
			if(wndFloat = new TFloatWindow(ctx)) {
				ATLASSERT(szCaption[0]);
				wndFloat->Create(m_hWnd, rcDefault, szCaption);
				ATLASSERT(::IsWindow(wndFloat->m_hWnd));
				ctx->hwndFloated = *wndFloat;

				//::SetParent(ctx->hwndChild, ctx->hwndDocked); set later depending on context
			}
		}

      // Add Context to master list
      if(wndFloat)
			return m_map.Add(ctx);
		else {
			delete ctx; // destructor takes care of windows
			return FALSE;
		}
   }

	// ~nab: i don't see in what context this permanent removal would be useful...
   BOOL RemoveWindow(HWND hWnd)
   {
      ATLASSERT(::IsWindow(hWnd));

      DOCKCONTEXT* pCtx = _GetContext(hWnd);
      ATLASSERT(pCtx);
      if( pCtx == NULL ) return FALSE;

		// remove from view
      T* pT = static_cast<T*>(this);
		if( IsFloating(pCtx->Side) ) return pT->_UnFloatWindow(pCtx);
      else if( IsDocked(pCtx->Side) ) return pT->_UnDockWindow(pCtx);

      // Remove context from map in case of repaint
      m_map.Remove(pCtx);

      ATLASSERT(::IsWindow(pCtx->hwndOrigPrnt));
      ::SetParent( pCtx->hwndChild, pCtx->hwndOrigPrnt );

      // Destroy context. Allocated in AddWindow()...
      delete pCtx; // destroys windows too

      return TRUE;
   }

	// iRequestedSize=-1 if you want a hidden pane (placeholder)
   BOOL DockWindow(HWND hWnd, short Side, int iRequestedSize = 0)
   {
      T* pT = static_cast<T*>(this);
      DOCKCONTEXT* pCtx = _GetContext(hWnd);
      ATLASSERT(pCtx);
      if( pCtx == NULL ) return FALSE;
      //::ShowWindow(pCtx->hwndChild, SW_SHOWNOACTIVATE);
      if( Side == DOCK_LASTKNOWN ) Side = pCtx->LastSide;
		ATLASSERT(IsDocked(Side));
      if( !IsDocked(Side) ) return FALSE;
		ATLASSERT(m_panes[Side].m_hWnd);
		if(!m_panes[Side].m_hWnd)
			return FALSE;
      return pT->_DockWindow(pCtx, Side, iRequestedSize, FALSE);
   }

   BOOL FloatWindow(HWND hWnd, const RECT& rc, BOOL bShow = TRUE)
   {
      T* pT = static_cast<T*>(this);
      DOCKCONTEXT* pCtx = _GetContext(hWnd);
      ATLASSERT(pCtx);
      if( pCtx == NULL ) return FALSE;
      //::ShowWindow(pCtx->hwndChild, SW_SHOWNOACTIVATE);
      pCtx->rcWindow = rc;
		if(rc.right-rc.left < DEFAULT_FLOAT_SIZE)
			pCtx->rcWindow.right = pCtx->rcWindow.left + DEFAULT_FLOAT_SIZE;
		if(rc.bottom-rc.top < DEFAULT_FLOAT_SIZE)
			pCtx->rcWindow.bottom = pCtx->rcWindow.top + DEFAULT_FLOAT_SIZE;
		// @@@ check rectangle is within some monitor?

      BOOL ret = pT->_FloatWindow(pCtx);
		if(!bShow)
			HideWindow(hWnd);
		return ret;
   }

	// helper to get client hwnd from dock pane ID
	HWND DockIDToHWND(UINT id)
	{
		int i, nPanes = m_map.GetSize();
		for(i=0; i<nPanes; i++)
			if(m_map[i]->idDockPane == id)
				return m_map[i]->hwndChild;

		return 0;
	}

	// Hide or show the pane without any radical measures
   BOOL HideWindow(HWND hWnd, BOOL bHide = TRUE)
   {
      T* pT = static_cast<T*>(this);
      DOCKCONTEXT* pCtx = _GetContext(hWnd);
      ATLASSERT(pCtx);
      if( pCtx == NULL ) return FALSE;

		if(bHide) {
			if( IsFloating(pCtx->Side) ) return pT->_UnFloatWindow(pCtx);
			else if( IsDocked(pCtx->Side) ) return pT->_UnDockWindow(pCtx, FALSE);
		}
		else {
			if( IsFloating(pCtx->LastSide) ) 
				return pT->_FloatWindow(pCtx);
			else if( IsDocked(pCtx->LastSide) ) 
				// we don't know of last size so use the default
				return pT->_DockWindow(pCtx, pCtx->LastSide, 0, TRUE);
			else
				ATLASSERT(0);
		}

      return TRUE;
   }

   void GetWindowState(HWND hWnd, int& DockState, RECT& rect) const
   {
      const DOCKCONTEXT* pCtx = _GetContext(hWnd);
      ATLASSERT(pCtx);
      if( pCtx == NULL ) return;
      DockState = pCtx->Side;
      ::GetWindowRect(::GetParent(hWnd), &rect);
   }

   int GetPaneSize(short Side) const
   {
      ATLASSERT(IsDocked(Side));
      int cy = m_panes[Side].m_cy;
      if( cy == 0 && m_panes[Side].m_hWnd) 
			cy = m_panes[Side].m_cyOld;
      return cy;
   }

	// resize the whole dock side
   void SetPaneSize(short Side, int iSize, BOOL bUpdate=TRUE)
   {
      ATLASSERT(IsDocked(Side));
		ATLASSERT(m_panes[Side].m_hWnd);
      if( iSize == 0 ) iSize = DEFAULT_DOCKPANE_SIZE;
      if( iSize < MIN_DOCKPANE_SIZE ) iSize = MIN_DOCKPANE_SIZE;
		if(iSize == m_panes[Side].m_cy)
			return;
      m_panes[Side].m_cy = ( m_panes[Side].CountVisiblePanes() == 0 ? 0 : iSize );
      m_panes[Side].m_cyOld = iSize;
      T* pT = static_cast<T*>(this);
		if(bUpdate && m_panes[Side].m_hWnd && m_panes[Side].m_cy)
			pT->UpdateLayout();
   }

#if 0 // ~nab: i think this is to keep all titlebars active at all times (?)
   void EnableModeless(BOOL bEnable)
   {
      for( int i = 0; i < m_map.GetSize(); i++ ) {
			if(IsFloating(m_map[i]->Side)) {
				ATLASSERT( ::IsWindowVisible(m_map[i]->hwndFloated) );
				::EnableWindow(m_map[i]->hwndFloated, bEnable);
			}
      }
   }
#endif

   // Call this method from an message idle-handler (such as the
   // WTL CIdleHandler::OnIdle() handler). It makes sure to
   // keep the docked panes' caption updated with respect to current
   // keyboard focus.
   virtual BOOL OnIdle()
   {
      HWND hwndFocus = ::GetFocus();
      HWND hwndCurrent = NULL;
		int i, nPanes = m_map.GetSize();

#ifdef _DEBUG
      for(i = 0; i < nPanes; i++ )
			if((m_map[i]->dwFlags & (DCK_TITLETEXT|DCK_GRADIENTEXT)) == 
				(DCK_TITLETEXT|DCK_GRADIENTEXT))
				break;
		ATLASSERT(i<nPanes); // no point calling this if you have no DCK_GRADIENTEXT!
#endif

      for(i = 0; i < nPanes; i++ ) {
         const DOCKCONTEXT* pCtx = m_map[i];
         if( IsDocked(pCtx->Side) && ::IsChild(pCtx->hwndDocked, hwndFocus) ) {
				hwndCurrent = pCtx->hwndDocked;
				break;
			}
      }

		const DOCKCONTEXT* pCtx = 0;
		HWND tmp = DockIDToHWND(m_idLastActive), s_hwndLastActive = 0;
		if(tmp) {
			pCtx = _GetContext(tmp);
			s_hwndLastActive = pCtx->hwndDocked;
		}

      if( s_hwndLastActive != hwndCurrent ) {
         if( s_hwndLastActive ) {
				ATLASSERT(pCtx);
				if(IsDocked(pCtx->Side) && (pCtx->dwFlags & DCK_GRADIENTEXT)) {
					ATLASSERT(::IsWindowVisible(s_hwndLastActive));
					// $TSEK the client isn't repainted, otherwise devise WM_DOCK_DOGRIPPER
					::InvalidateRect(s_hwndLastActive, NULL, TRUE);
				}
			}
         if( ::IsWindow(hwndCurrent) ) {
				ATLASSERT(hwndCurrent==m_map[i]->hwndDocked);
				m_idLastActive = m_map[i]->idDockPane;
				if(m_map[i]->dwFlags & DCK_GRADIENTEXT)
					::InvalidateRect(hwndCurrent, NULL, TRUE);
			}
			else
				m_idLastActive = -1;
      }
      return TRUE;
   }

	// call this from a keyboard handler dealing e.g. with TAB/Shift+TAB
	void FocusNextPane(BOOL bNext = TRUE)
	{
		HWND hwndFocus = ::GetFocus();
      int i, nPanes = m_map.GetSize();
      for(i = 0; i < nPanes; i++ ) {
         const DOCKCONTEXT* pCtx = m_map[i];
			if(pCtx->Side == DOCK_HIDDEN)
				continue;

         if( hwndFocus==pCtx->hwndChild || ::IsChild(pCtx->hwndChild, hwndFocus) )
				break;
      }

		if(i < nPanes) {
			ATLASSERT(m_map[i]->Side != DOCK_HIDDEN);
			// move to next or previous, possibly with cycling
			int step = bNext ? 1 : -1;
			while(1) {
				i += step;
				if(i >= nPanes)
					i=0;
				else if(i<0)
					i = nPanes - 1;

				if(m_map[i]->Side != DOCK_HIDDEN)
					break;
			}

			// at worst we'll reach back on the same pane
			::SetFocus(m_map[i]->hwndChild);
		}
	}

	// registry state persistence
	/* STATE FORMAT
	 * Each dock side has a number of docked bars, and a comma separated list of bar IDs
	 * Similar information for the floating bars
	 * Then each bar stores 5 numbers in a comma separated list: visible, and the rectangle
	 */
#define DOCKREG_SECTION _T("DockBars")
#define DOCKREG_NUM _T("Num%d")
#define DOCKREG_PANES _T("Panes%d")
#define DOCKREG_BAR _T("Bar%d")
#define DOCKREG_LOCKED _T("Locked")

	// load state from registry and provide defaults for new bars
	// defstate should be an array of m_map.GetSize() elements
#ifdef DOCK_DUAL_REGINI
	int LoadBarState(CRegKeyEx& rkParent, DOCK_DEFBARSTATE* defstate)
#else
	int LoadBarState(LPCTSTR lpstrRegKey, DOCK_DEFBARSTATE* defstate)
#endif
	{
		ATLASSERT(defstate);
		int nBars = m_map.GetSize();

		int i, loaded;
#ifdef DOCK_DUAL_REGINI
		loaded = _LoadBarState(rkParent);
#else
		loaded = _LoadBarState(lpstrRegKey);
#endif

		if(loaded < nBars) {
			ATLASSERT(!loaded); // fresh installation
			// provide defaults for all remaining bars
			RECT rcnil = {0};
			for(i=0; i<nBars; i++) {
				HWND hw = DockIDToHWND(defstate[i].idDockPane);
				if(IsDockbarLoaded(hw))
					continue;

				short dock = defstate[i].Side;
				if(dock < DOCK_FLOAT) {
					ATLASSERT(IsDocked(dock));
					DockWindow(hw, dock, defstate[i].bVisible ? 0 : -1);
				}
				else
					FloatWindow(hw, rcnil, defstate[i].bVisible);
			}
		}

#ifdef _DEBUG
		// make sure all our bars were initialized
		for(i=0; i<nBars; i++)
			ATLASSERT(IsDockbarLoaded(DockIDToHWND(defstate[i].idDockPane)));
#endif

		return loaded;
	}

#ifdef DOCK_DUAL_REGINI
	int _LoadBarState(CRegKeyEx& rkParent)
#else
	int _LoadBarState(LPCTSTR lpstrRegKey)
#endif
	{
		ATLASSERT(IsWindow()); // after creation
		ATLASSERT(m_map.GetSize()); // after AddWindow all bars

		LONG lRet;
#ifdef DOCK_DUAL_REGINI
		CRegKeyEx rk;
#else
		ATLASSERT(*lpstrRegKey);
		CRegKey rkParent, rk;
		lRet = rkParent.Open(HKEY_CURRENT_USER, lpstrRegKey);
		if(lRet != ERROR_SUCCESS)
			return 0;
#endif

		lRet = rk.Open(rkParent, DOCKREG_SECTION);
		if(lRet != ERROR_SUCCESS)
			return 0;

		DWORD bLocked = 0;
#if (_ATL_VER >= 0x0700)
		lRet = rk.QueryDWORDValue(DOCKREG_LOCKED, bLocked);
#else
		lRet = rk.QueryValue(bLocked, DOCKREG_LOCKED);
#endif

		short dock;
		int i, k, id, nAdded = 0;
		TCHAR buf[100], str[1024], dimension[50];
		for(dock = DOCK_LEFT; dock <= DOCK_FLOAT; dock++) {
			if(dock < DOCK_FLOAT && 0==m_panes[dock].m_hWnd)
				continue; // no docking here

			wsprintf(buf, DOCKREG_NUM, dock);
			DWORD nBars = 0;
#if (_ATL_VER >= 0x0700)
			lRet = rk.QueryDWORDValue(buf, nBars);
#else
			lRet = rk.QueryValue(nBars, buf);
#endif
			ATLASSERT(ERROR_SUCCESS==lRet || 0==nBars);
			if(!nBars)
				continue;

			// read bar IDs
			wsprintf(buf, DOCKREG_PANES, dock);
			DWORD dwCount = sizeof(str); // bytes!
			// for convenience use REG_MULTI_SZ
#if (_ATL_VER >= 0x0700) || defined(DOCK_DUAL_REGINI)
			// an easier solution would be GetProfileArray
			dwCount /= sizeof(TCHAR);
			lRet = rk.QueryMultiStringValue(buf, str, &dwCount);
			ATLASSERT(dwCount == nBars*6+1); // 5 digit numbers
#else
			lRet = rk.QueryValue(str, buf, &dwCount);
			ATLASSERT(dwCount/sizeof(TCHAR) == nBars*6+1); // 5 digit numbers
#endif
			if(lRet != ERROR_SUCCESS)
				continue;

			int nExtent = 0; // dock side width (or height)
			LPCTSTR prb = str;
			for(i=0; i<(int)nBars; i++, prb += lstrlen(prb) + 1 /*next ID*/) {
				ATLASSERT(*prb);
				if(0==*prb)
					break; // corrupt

				id = _ttoi(prb);
				ATLASSERT(id >= ATL_IDW_DOCKPANE_FIRST && id < ATL_IDW_PANE_FIRST);

				HWND hw = DockIDToHWND(id);
				ATLASSERT(hw);
				if(!hw)
					continue;
				const DOCKCONTEXT* ctx = _GetContext(hw);
				ATLASSERT(ctx->LastSide == DOCK_HIDDEN); // don't dock or float before

				wsprintf(buf, DOCKREG_BAR, id);
				dwCount = sizeof(dimension);
#if (_ATL_VER >= 0x0700) || defined(DOCK_DUAL_REGINI)
				dwCount /= sizeof(TCHAR);
				lRet = rk.QueryMultiStringValue(buf, dimension, &dwCount);
				ATLASSERT(dwCount >= 11); // 5 numbers
#else
				lRet = rk.QueryValue(dimension, buf, &dwCount);
				ATLASSERT(dwCount/sizeof(TCHAR) >= 11); // 5 numbers
#endif
				if(lRet != ERROR_SUCCESS)
					continue;

				LPCTSTR p2 = dimension;
				BOOL bVisible = _ttoi(p2);
				RECT rect;
				LPLONG plong = (LPLONG)&rect;
				ATLASSERT(sizeof(rect)==4*sizeof(LONG));
				for(k=0; k<4; k++) {
					p2 += lstrlen(p2) + 1;
					ATLASSERT(*p2);
					*plong++ = _ttoi(p2);
				}
				ATLASSERT(0==bVisible || !IsRectEmpty(&rect));

				// dwFlags DCK_xxx aren't persisted, these are fixed at creation time
				// one exception is the locked state
				if(bLocked) {
					ATLASSERT(!(_GetContext(hw)->dwFlags & DCK_NOGRIPPER));
					_GetContext(hw)->dwFlags |= DCK_NOGRIPPER;
				}

				// dock or float
				if(dock < DOCK_FLOAT) {
					ATLASSERT(IsDocked(dock));
					int size;
					if(!bVisible) {
						size = -1;
						k = 0;
					}
					else if(IsDockedVertically(dock)) {
						size = rect.bottom - rect.top;
						k = rect.right - rect.left;
					}
					else {
						size = rect.right - rect.left;
						k = rect.bottom - rect.top;
					}

					if(k && k != nExtent) {
						ATLASSERT(!nExtent); // all rects should be equal
						nExtent = k;
						ATLASSERT(m_panes[dock].m_cxySplitter == DEFAULT_SPLITTER_SIZE);
						// take into account the dock site's 'non-client' area
						SetPaneSize(dock, nExtent+DEFAULT_SPLITTER_SIZE, FALSE);
					}

					DockWindow(hw, dock, size);
				}
				else
					FloatWindow(hw, rect, bVisible);

				ATLASSERT(ctx->LastSide != DOCK_HIDDEN);
				nAdded++;
			}
		}

		return nAdded; // return # of panes loaded
	}

#ifdef DOCK_DUAL_REGINI
	BOOL SaveBarState(CRegKeyEx& rkParent)
#else
	BOOL SaveBarState(LPCTSTR lpstrRegKey)
#endif
	{
		ATLASSERT(m_map.GetSize());

		LONG lRet;
#ifdef DOCK_DUAL_REGINI
		CRegKeyEx rk;
#else
		ATLASSERT(*lpstrRegKey);
		CRegKey rkParent, rk;
		lRet = rkParent.Create(HKEY_CURRENT_USER, lpstrRegKey);
		if(lRet != ERROR_SUCCESS)
			return FALSE;
#endif
		lRet = rk.Create(rkParent, DOCKREG_SECTION);
		if(lRet != ERROR_SUCCESS)
			return FALSE;

		TCHAR buf[100], str[1024];
		int nTotal = m_map.GetSize();
		for(short dock = DOCK_LEFT; dock <= DOCK_FLOAT; dock++) {
			int i, k, nBars;
			LPTSTR prb = str;
			if(IsDocked(dock))
				nBars = m_panes[dock].m_map.GetSize();
			else {
				// figure out floated bars from main array
				nBars = 0;
				for(i = 0; i < nTotal; i++) {
					if(m_map[i]->LastSide == DOCK_FLOAT) {
						nBars++;
						wsprintf(buf, _T("%d"), m_map[i]->idDockPane);
						lstrcpy(prb, buf);
						prb += lstrlen(buf)+1;
					}
				}
			}

			wsprintf(buf, DOCKREG_NUM, dock);
#if (_ATL_VER >= 0x0700)
			lRet = rk.SetDWORDValue(buf, nBars);
#else
			lRet = rk.SetValue(nBars, buf);
#endif
			ATLASSERT(lRet == ERROR_SUCCESS);
			if(!nBars)
				continue;

			if(IsDocked(dock)) {
				// store bar IDs for this dock side
				for(i = 0; i < nBars; i++) {
					wsprintf(buf, _T("%d"), m_panes[dock].m_map[i]->idDockPane);
					lstrcpy(prb, buf);
					prb += lstrlen(buf)+1;
				}
			}
			*prb++ = 0; // double terminator
			ATLASSERT(prb - str < sizeof(str)/sizeof(str[0]));

			wsprintf(buf, DOCKREG_PANES, dock);
#if (_ATL_VER >= 0x0700) || defined(DOCK_DUAL_REGINI)
			lRet = rk.SetMultiStringValue(buf, str);
#else
			// no way to write REG_MULTI_SZ in CRegKey
			lRet = RegSetValueEx(rk.m_hKey, buf, 0, REG_MULTI_SZ, (LPBYTE)str, (prb-str)*sizeof(TCHAR) );
#endif
			ATLASSERT(lRet == ERROR_SUCCESS);

			// finally store individual bar information
			for(i = 0; i < nTotal; i++) {
				if(m_map[i]->LastSide == dock) {
					prb = str;
					wsprintf(buf, _T("%d"), m_map[i]->Side != DOCK_HIDDEN);
					lstrcpy(prb, buf);
					LPLONG plong = (LPLONG)&m_map[i]->rcWindow;
					for(k=0; k<4; k++) {
						prb += lstrlen(prb)+1;
						wsprintf(buf, _T("%d"), *plong++);
						lstrcpy(prb, buf);
					}
					prb += lstrlen(prb)+1;
					*prb++ = 0; // double terminator

					wsprintf(buf, DOCKREG_BAR, m_map[i]->idDockPane);
#if (_ATL_VER >= 0x0700) || defined(DOCK_DUAL_REGINI)
					lRet = rk.SetMultiStringValue(buf, str);
#else
					lRet = RegSetValueEx(rk.m_hKey, buf, 0, REG_MULTI_SZ, (LPBYTE)str, (prb-str)*sizeof(TCHAR));
#endif
					ATLASSERT(lRet == ERROR_SUCCESS);
				}
			}
		}

		DWORD bLocked = IsLocked();
#if (_ATL_VER >= 0x0700)
		lRet = rk.SetDWORDValue(DOCKREG_LOCKED, bLocked);
#else
		lRet = rk.SetValue(bLocked, DOCKREG_LOCKED);
#endif

		return TRUE;
	}

	// if any dockbars weren't initialized, check which ones loaded successfully from persistent store
	// manually dock/float any windows that fail this test!
	BOOL IsDockbarLoaded(HWND hWnd)
	{
		DOCKCONTEXT* ctx = _GetContext(hWnd);
		ATLASSERT(ctx);
		if(!ctx)
			return FALSE;

		return ctx->LastSide != DOCK_HIDDEN;
	}

	// lock panes in place. They can't be moved but they can be resized
	void Lock(BOOL lock = TRUE)
	{
		if(IsLocked() == lock)
			return;

		// flip all dock sites DCK_NOGRIPPER flag and redraw
		int i, nTotal = m_map.GetSize();
		for(i=0; i<nTotal; i++) {
			if(lock)
				m_map[i]->dwFlags |= DCK_NOGRIPPER;
			else
				m_map[i]->dwFlags &= ~DCK_NOGRIPPER;

			if(IsDocked(m_map[i]->Side))
				::SendMessage(m_map[i]->hwndDocked, WM_DOCK_UPDATELAYOUT,0,0);
		}
	}

	BOOL IsLocked()
	{
		// we expect all panes to be the same with or without titlebar
		int i, nTotal = m_map.GetSize();
		for(i=0; i<nTotal; i++)
			if(m_map[i]->dwFlags & DCK_NOGRIPPER)
				return TRUE;

		return FALSE;
	}

   // Message handlers
protected:
   LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
		BOOL numdock = 0;
      for( int i = 0; i < 4; i++ ) {
         m_panes[i].m_Side = (short) i;
			if(m_dwExtStyle & (1<<i))
				continue; // implement DCK_EX_NOLEFT etc

         m_panes[i].Create(m_hWnd, rcDefault, NULL, WS_CHILD|WS_VISIBLE);
			numdock++;
      }
		ATLASSERT(numdock); // surely one?
		if(!numdock)
			return -1;

      m_sizeBorder.cx = ::GetSystemMetrics(SM_CXEDGE);
      m_sizeBorder.cy = ::GetSystemMetrics(SM_CYEDGE);
      return 0;
   }

   LRESULT OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
		// we get some silly WM_SIZE during destruction which we can't handle
		// i think they are generated by the rebar during its destruction
		m_bNoUpdates = 1;

      int i = m_map.GetSize();
		while(i--) {
			ATLASSERT(m_map[i]->LastSide != DOCK_HIDDEN); // not in limbo?
			delete m_map[i];
		}
      return 0;
   }

   LRESULT OnEraseBackground(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      return 1; // handled, no background painting needed
   }

   LRESULT OnSize(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
   {
      T* pT = static_cast<T*>(this);
      pT->UpdateLayout();
      return 0;
   }

   // Custom defined messages

   LRESULT OnQueryRect(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      // wParam: Side
      // lParam: LPRECT
      LPRECT prc = (LPRECT) lParam;
      ATLASSERT(prc);
      switch( (short) wParam ) {
      case DOCK_LEFT:
      case DOCK_TOP:
      case DOCK_RIGHT:
      case DOCK_BOTTOM:
			ATLASSERT(m_panes[wParam].m_hWnd);
         ::GetWindowRect(m_panes[wParam], prc);
         break;
      case DOCK_INFO_CHILD: // only called for client
         ::GetWindowRect(m_hwndClient, prc);
         break;
      default:
         ATLASSERT(false);
      }
      return 0;
   }

	// check where we should be docking
   LRESULT OnQueryTrack(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
   {
      // lParam: DOCK_TRACKINFO*
      DOCK_TRACKINFO* pTI = (DOCK_TRACKINFO*) lParam;
      ATLASSERT(pTI);
      POINT& pt = pTI->ptPos;
      RECT rc;

		BOOL canFloat = (pTI->pCtx->dwFlags & DCK_NOFLOAT) == 0 ;
		BOOL mustFloat = canFloat && 
	      // Pressing CTRL key gives default floating
			( ::GetKeyState(VK_CONTROL) < 0 ) ;

		::GetWindowRect(pTI->hWnd, &rc);
		if(!mustFloat) {
			RECT saved = rc;
#if 0 // confusing exception
			// Are we perhaps hovering over the tracked window?
			if( ::PtInRect(&rc, pt) ) {
				pTI->rc = rc;
				::OffsetRect(&pTI->rc, pt.x - pTI->ptStart.x, pt.y - pTI->ptStart.y);
				pTI->Side = pTI->pCtx->Side;
				return 0;
			}
#endif
			// Or is the point inside one of the other docking areas?
			int i;
			for( i = 0; i < 4; i++ ) {
				if( pTI->pCtx->dwFlags & (1<<i) ) continue; // DCK_NOxxx flag?
				ATLASSERT(m_panes[i].m_Side==i);
				if( m_panes[i].m_cy == 0) {
					if(!m_panes[i].m_hWnd)
						continue;

					// Simulate docking areas that are currently invisible
					::GetWindowRect(m_hWnd, &rc);
					switch( m_panes[i].m_Side ) {
					case DOCK_LEFT:   rc.right = rc.left + DEFAULT_DOCKPANE_SIZE; break;
					case DOCK_RIGHT:  rc.left = rc.right - DEFAULT_DOCKPANE_SIZE; break;
					case DOCK_TOP:    rc.bottom = rc.top + DEFAULT_DOCKPANE_SIZE; break;
					case DOCK_BOTTOM: rc.top = rc.bottom - DEFAULT_DOCKPANE_SIZE; break;
					}
				}
				else {
					ATLASSERT(m_panes[i].m_hWnd);
					::GetWindowRect(m_panes[i], &rc);
				}
				if( ::PtInRect(&rc, pt) ) 
					break;
			}

			if(i < 4 || 0==canFloat )
			{
				if(i==4) {
					//find the first available dock
					for(i=0; i<4; i++)
						if(m_panes[i].m_hWnd)
							break;
				}

				pTI->Side = (short) i;
				pTI->rc = rc;
				return 0;
			}

			rc = saved;
		}

      // But by default, we're floating in the skies

      ::SetRect(&pTI->rc, rc.left, rc.top, rc.left + pTI->pCtx->rcWindow.right-pTI->pCtx->rcWindow.left, 
			rc.top + pTI->pCtx->rcWindow.bottom-pTI->pCtx->rcWindow.top);
      ::OffsetRect(&pTI->rc, pt.x - pTI->ptStart.x, pt.y - pTI->ptStart.y);
      pTI->Side = DOCK_FLOAT;
      return 0;
   }

   LRESULT OnDock(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      T* pT = static_cast<T*>(this);
		DOCK_TRACKINFO* ptr = (DOCK_TRACKINFO*)lParam;
		// try to get a dock position
      pT->_DockWindow(ptr->pCtx, (short) wParam, 0, TRUE, m_panes[wParam].DockPosition(ptr));
      return 0;
   }

   LRESULT OnUnDock(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
   {
      T* pT = static_cast<T*>(this);
      pT->_UnDockWindow((DOCKCONTEXT*) lParam);
      return 0;
   }

   LRESULT OnFloat(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
   {
      T* pT = static_cast<T*>(this);
		DOCK_TRACKINFO* ptr = (DOCK_TRACKINFO*)lParam;
      pT->_FloatWindow(ptr->pCtx);
      return 0;
   }

   LRESULT OnUnFloat(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
   {
      T* pT = static_cast<T*>(this);
      pT->_UnFloatWindow((DOCKCONTEXT*) lParam);
      return 0;
   }

   LRESULT OnClientClose(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM lParam, BOOL& /*bHandled*/)
   {
      DOCKCONTEXT* pCtx = (DOCKCONTEXT*) lParam;

		// ~nab: change of plans: notify parent and if unhandled just hide the window
		NMHDR nmh = { pCtx->hwndChild, pCtx->idDockPane, WM_DOCK_CLIENT_CLOSE };
		if(::SendMessage(GetParent(), WM_NOTIFY, nmh.idFrom, (LPARAM) &nmh)==0)
			HideWindow(pCtx->hwndChild);

#if 0
      // Make sure to bring focus back to this app!
      ::SetForegroundWindow(m_hWnd);
      SetFocus();
      // Hide or destroy view
      if( (m_dwExtStyle & DCK_EX_DESTROYONCLOSE) != 0 ) {
         HWND hWnd = pCtx->hwndChild;
         // Commit suicide
         RemoveWindow(hWnd);
         ::DestroyWindow(hWnd);
      }
      else {
         // Send the actual dock/float message instead
         SendMessage((UINT) wParam, 0, (LPARAM) pCtx);
         // HACK: Send notification to child (note the funky message type)
         ::SendMessage(pCtx->hwndChild, WM_COMPACTING, 0, 0L); //who is dealing with this?
      }
#endif
      return 0;
   }

   LRESULT OnRepositionWindow(UINT /*uMsg*/, WPARAM wParam, LPARAM lParam, BOOL& /*bHandled*/)
   {
      T* pT = static_cast<T*>(this);

		DOCK_TRACKINFO* ptr = (DOCK_TRACKINFO*)lParam;
		// try to get a dock position
      pT->_RepositionWindow(ptr->pCtx, m_panes[wParam].DockPosition(ptr));
      return 0;
   }

   // Overridables

   void UpdateLayout()
   {
		if(m_bNoUpdates)
			return;

      RECT rect;   
      GetClientRect(&rect);
		if(rect.bottom==rect.top)
			return; // try later
      RECT rcClient = rect;

		// check we have room to see through the client a little bit
		BOOL baxalo = FALSE;
		int aux = rect.bottom - rect.top;
		if(m_panes[DOCK_TOP].m_cy + m_panes[DOCK_BOTTOM].m_cy + MIN_DOCKPANE_SIZE >= aux) {
			aux /= 3;
		   SetPaneSize(DOCK_TOP, aux, FALSE);
		   SetPaneSize(DOCK_BOTTOM, aux, FALSE);
			baxalo = 1;
		}
		aux = rect.right - rect.left;
		if(m_panes[DOCK_LEFT].m_cy + m_panes[DOCK_RIGHT].m_cy + MIN_DOCKPANE_SIZE >= aux ) {
			aux /= 3;
		   SetPaneSize(DOCK_LEFT, aux, FALSE);
		   SetPaneSize(DOCK_RIGHT, aux, FALSE);
			baxalo = 1;
		}

		SIZE curr = {rect.right-rect.left, rect.bottom-rect.top};
		if(0==baxalo && m_sizeLast.cx) {
			// spread the resize proportionally among the panes
			aux = curr.cx - m_sizeLast.cx;
			if(aux) {
				aux = m_panes[DOCK_LEFT].m_cy;
				if(aux)
					// MulDiv rounds to the closest integer
				   SetPaneSize(DOCK_LEFT, MulDiv(aux, curr.cx, m_sizeLast.cx), FALSE);
				aux = m_panes[DOCK_RIGHT].m_cy;
				if(aux)
				   SetPaneSize(DOCK_RIGHT, MulDiv(aux,curr.cx,m_sizeLast.cx), FALSE);
			}
			aux = curr.cy - m_sizeLast.cy;
			if(aux) {
				ATLASSERT(m_sizeLast.cy);
				aux = m_panes[DOCK_TOP].m_cy;
				if(aux)
				   SetPaneSize(DOCK_TOP, MulDiv(aux,curr.cy,m_sizeLast.cy), FALSE);
				aux = m_panes[DOCK_BOTTOM].m_cy;
				if(aux)
				   SetPaneSize(DOCK_BOTTOM, MulDiv(aux,curr.cy,m_sizeLast.cy), FALSE);
			}
		}
		m_sizeLast = curr;

      if( m_panes[DOCK_TOP].m_cy > 0 ) {
         ::SetWindowPos(m_panes[DOCK_TOP], NULL, rect.left, rect.top, rect.right - rect.left, m_panes[DOCK_TOP].m_cy, SWP_NOZORDER | SWP_NOACTIVATE);
         rcClient.top += m_panes[DOCK_TOP].m_cy;
      }
      if( m_panes[DOCK_LEFT].m_cy > 0 ) {
         ::SetWindowPos(m_panes[DOCK_LEFT], NULL, rect.left, rcClient.top, m_panes[DOCK_LEFT].m_cy, rect.bottom - rcClient.top, SWP_NOZORDER | SWP_NOACTIVATE);
         rcClient.left += m_panes[DOCK_LEFT].m_cy;
      }
      if( m_panes[DOCK_RIGHT].m_cy > 0 ) {
         //int left = max( (int) rect.right - m_panes[DOCK_RIGHT].m_cy, (int) rcClient.left );
         ::SetWindowPos(m_panes[DOCK_RIGHT], NULL, rect.right - m_panes[DOCK_RIGHT].m_cy, rcClient.top, m_panes[DOCK_RIGHT].m_cy, rect.bottom - rcClient.top, SWP_NOZORDER | SWP_NOACTIVATE);
         rcClient.right -= m_panes[DOCK_RIGHT].m_cy;
      }
      if( m_panes[DOCK_BOTTOM].m_cy > 0 ) {
         //int top = max( (int) rcClient.bottom - m_panes[DOCK_BOTTOM].m_cy, (int) rcClient.top );
         ::SetWindowPos(m_panes[DOCK_BOTTOM], NULL, rcClient.left, rect.bottom-m_panes[DOCK_BOTTOM].m_cy, rcClient.right - rcClient.left, m_panes[DOCK_BOTTOM].m_cy, SWP_NOZORDER | SWP_NOACTIVATE);
         rcClient.bottom -= m_panes[DOCK_BOTTOM].m_cy;
      }

      if( ::IsWindow(m_hwndClient) ) {
#if 0
         // Adjust borders around docking panes
         DWORD dwExtStyle = (DWORD)::GetWindowLong(m_hwndClient, GWL_EXSTYLE);
         bool bClientEdge = ((dwExtStyle & WS_EX_CLIENTEDGE) != 0);
         if( bClientEdge ) {
				// this correction is required because our edges already look like clientedge?!
            if( m_panes[DOCK_TOP].m_cy > 0 ) rcClient.top -= m_sizeBorder.cy;
            if( m_panes[DOCK_LEFT].m_cy > 0 ) rcClient.left -= m_sizeBorder.cx;
            if( m_panes[DOCK_RIGHT].m_cy > 0 ) rcClient.right += m_sizeBorder.cx;
            if( m_panes[DOCK_BOTTOM].m_cy > 0 ) rcClient.bottom += m_sizeBorder.cy;
         }
#endif
         // Map client rectangle to windows's coord system
         ::MapWindowPoints(m_hWnd, ::GetParent(m_hwndClient), (LPPOINT)&rcClient, sizeof(rcClient)/sizeof(POINT));
         ::SetWindowPos(m_hwndClient, NULL, rcClient.left, rcClient.top, rcClient.right - rcClient.left, rcClient.bottom - rcClient.top, SWP_NOZORDER | SWP_NOACTIVATE);
      }
   }

   BOOL _DockWindow(DOCKCONTEXT* ctx, short Side, int iRequestedSize, BOOL bSetFocus, int slot = -1)
   {
      ATLASSERT(ctx);
      ATLASSERT(IsDocked(Side));
      ATLASSERT(ctx->Side==DOCK_HIDDEN); // Undock before redock
		ATLASSERT(m_panes[Side].m_hWnd);

      if( !IsDocked(Side) ) return FALSE;
      if( ctx->Side != DOCK_HIDDEN ) return FALSE;
      bool bVertical = IsDockedVertically(Side);
      // Make up a new panel size
      if( iRequestedSize == 0 ) {
         RECT rc;
         ::GetClientRect(m_panes[Side], &rc);
         iRequestedSize = bVertical ? rc.bottom - rc.top : rc.right - rc.left;
			ATLASSERT(iRequestedSize >= 0);
			int nPanes = m_panes[Side].CountVisiblePanes();
         if( nPanes > 0 ) iRequestedSize /= (nPanes+1);
      }
      //if( iRequestedSize < 10 ) iRequestedSize = 0; minimum sizes are enforced in UpdateLayout
      // Set the new size of the pane (subject to readjustment)
      ::SetRectEmpty(&ctx->rcWindow);
      (bVertical ? ctx->rcWindow.bottom : ctx->rcWindow.right) = iRequestedSize;
      // Signal that we would like to retain this size after
      // first being laid out...
      ctx->bKeepSize = iRequestedSize > 0;
		int oldcy = m_panes[Side].m_cy; //GetPaneSize(Side);
      // Dock
      m_panes[Side].DockWindow(ctx, slot);
      if( bSetFocus ) ::SetFocus(ctx->hwndChild);

		if(oldcy != m_panes[Side].m_cy)
			PostMessage(WM_DOCK_UPDATELAYOUT);
      return TRUE;
   }

   BOOL _FloatWindow(DOCKCONTEXT* ctx)
   {
      ATLASSERT(ctx);
      ATLASSERT(ctx->Side==DOCK_HIDDEN); // Undock before float
		ATLASSERT( (ctx->dwFlags & DCK_NOFLOAT) == 0 );

      if( ctx->Side != DOCK_HIDDEN ) return FALSE;
      ctx->Side = DOCK_FLOAT;
      ctx->LastSide = DOCK_FLOAT;
      ::SetParent(ctx->hwndChild, ctx->hwndFloated);
      ::SetWindowPos(ctx->hwndFloated, NULL, ctx->rcWindow.left, ctx->rcWindow.top, ctx->rcWindow.right - ctx->rcWindow.left, ctx->rcWindow.bottom - ctx->rcWindow.top, SWP_NOZORDER);
      ::SendMessage(ctx->hwndFloated, WM_DOCK_UPDATELAYOUT, 0, 0L);
      ::ShowWindow(ctx->hwndFloated, SW_SHOWNORMAL);
      return TRUE;
   }

   BOOL _UnDockWindow(DOCKCONTEXT* ctx, BOOL bRemove=TRUE)
   {
      ATLASSERT(ctx);
      ATLASSERT(IsDocked(ctx->Side));
		ATLASSERT(m_panes[ctx->Side].m_hWnd);

      if( !IsDocked(ctx->Side) ) return FALSE;
		int oldcy = m_panes[ctx->Side].m_cy; //GetPaneSize(Side); -> or WM_DOCK_QUERYRECT

      m_panes[ctx->Side].UnDockWindow(ctx, bRemove);
      ctx->Side = DOCK_HIDDEN;

		if(oldcy != m_panes[ctx->Side].m_cy)
	      SendMessage(WM_DOCK_UPDATELAYOUT);
      return TRUE;
   }

   BOOL _UnFloatWindow(DOCKCONTEXT* ctx)
   {
      ATLASSERT(ctx);
      ATLASSERT(IsFloating(ctx->Side));
      if( !IsFloating(ctx->Side) ) return FALSE;
#ifdef _DEBUG
		RECT rc_;
		::GetWindowRect(ctx->hwndFloated, &rc_);
		ATLASSERT(EqualRect(&rc_, &ctx->rcWindow));
		//ATLASSERT(ctx->sizeFloat.cx == rc_.right-rc_.left && ctx->sizeFloat.cy == rc_.bottom-rc_.top);
#endif
      ::ShowWindow(ctx->hwndFloated, SW_HIDE);
      ctx->Side = DOCK_HIDDEN;
      return TRUE;
   }

   BOOL _RepositionWindow(DOCKCONTEXT* ctx, int iPos)
   {
      ATLASSERT(ctx);
      ATLASSERT(IsDocked(ctx->Side));
		ATLASSERT(m_panes[ctx->Side].m_hWnd);

      if( !IsDocked(ctx->Side) ) return FALSE;
      m_panes[ctx->Side].RepositionWindow(ctx,iPos);
      return TRUE;
   }

   DOCKCONTEXT* _GetContext(HWND hWnd) const
   {
      ATLASSERT(::IsWindow(hWnd));
      for( int i = 0; i < m_map.GetSize(); i++ ) if( m_map[i]->hwndChild == hWnd ) return m_map[i];
      ATLASSERT(!_T("Docking Window not found; use AddWindow() to add it"));
      return NULL;
   }
};

class CDockingWindow : public CDockingWindowImpl<CDockingWindow>
{
public:
   DECLARE_WND_CLASS_EX(_T("WTL_DockingWindow"), CS_HREDRAW|CS_VREDRAW|CS_DBLCLKS, NULL)
};


#endif // __ATL_DOCK_H__
