// MainFrm.h : interface of the CMainFrame class
//
/////////////////////////////////////////////////////////////////////////////

#if !defined(AFX_MAINFRM_H__5DBB058E_0908_442B_9ED4_DECBDE98E4CD__INCLUDED_)
#define AFX_MAINFRM_H__5DBB058E_0908_442B_9ED4_DECBDE98E4CD__INCLUDED_

#if _MSC_VER >= 1000
#pragma once
#endif // _MSC_VER >= 1000

class CMainFrame : public CFrameWindowImpl<CMainFrame>, public CUpdateUI<CMainFrame>,
		public CMessageFilter, public CIdleHandler
#ifdef DOCK_DUAL_REGINI
		, public CRegistryAccess
#endif
{
public:
	DECLARE_FRAME_WND_CLASS(NULL, IDR_MAINFRAME)

	CDockingView m_view; // real client

	CDockingWindow m_dock; // docking infrastructure
	CEdit m_bar1, m_bar2, m_bar3; 
	CListViewCtrl m_bar4;
#ifndef DOCK_DUAL_REGINI
	CString m_strRegistryKey;
#endif

	virtual BOOL PreTranslateMessage(MSG* pMsg)
	{
		if(CFrameWindowImpl<CMainFrame>::PreTranslateMessage(pMsg))
			return TRUE;

		return m_view.PreTranslateMessage(pMsg);
	}

	virtual BOOL OnIdle()
	{
		UIUpdateToolBar();
		UISetCheck(ID_VIEW_BAR1, ::IsWindowVisible(m_bar1));
      UISetCheck(ID_VIEW_BAR2, ::IsWindowVisible(m_bar2));
      UISetCheck(ID_VIEW_BAR3, ::IsWindowVisible(m_bar3));
      UISetCheck(ID_VIEW_BAR4, ::IsWindowVisible(m_bar4));

      UISetCheck(ID_VIEW_LOCKDOCKS, m_dock.IsLocked());
		m_dock.OnIdle(); // handle active titlebars

		return FALSE;
	}

	BEGIN_UPDATE_UI_MAP(CMainFrame)
		UPDATE_ELEMENT(ID_VIEW_TOOLBAR, UPDUI_MENUPOPUP)
		UPDATE_ELEMENT(ID_VIEW_STATUS_BAR, UPDUI_MENUPOPUP)
		UPDATE_ELEMENT(ID_VIEW_BAR1, UPDUI_MENUPOPUP)
      UPDATE_ELEMENT(ID_VIEW_BAR2, UPDUI_MENUPOPUP)
      UPDATE_ELEMENT(ID_VIEW_BAR3, UPDUI_MENUPOPUP)
      UPDATE_ELEMENT(ID_VIEW_BAR4, UPDUI_MENUPOPUP)
      UPDATE_ELEMENT(ID_VIEW_LOCKDOCKS, UPDUI_MENUPOPUP)
	END_UPDATE_UI_MAP()

	BEGIN_MSG_MAP(CMainFrame)
		MESSAGE_HANDLER(WM_CREATE, OnCreate)
		COMMAND_ID_HANDLER(ID_APP_EXIT, OnFileExit)
		COMMAND_ID_HANDLER(ID_FILE_NEW, OnFileNew)
		COMMAND_ID_HANDLER(ID_VIEW_TOOLBAR, OnViewToolBar)
		COMMAND_ID_HANDLER(ID_VIEW_STATUS_BAR, OnViewStatusBar)
		COMMAND_ID_HANDLER(ID_APP_ABOUT, OnAppAbout)
      MESSAGE_HANDLER(WM_DESTROY, OnDestroy)

		COMMAND_ID_HANDLER(ID_VIEW_LOCKDOCKS, OnLockDocks)
		COMMAND_RANGE_HANDLER(ID_NEXT_PANE, ID_PREV_PANE, OnSwitchDock)
		COMMAND_RANGE_HANDLER(ID_VIEW_BAR1, ID_VIEW_BAR4, OnViewBar)
      COMMAND_ID_HANDLER(ID_FILE_PRINT, OnAddItem) // tests for notifications
		//CHAIN_DOCK_CHILD_COMMANDS(m_bar1) //if we had message handling bars...
		NOTIFY_CODE_HANDLER(LVN_GETDISPINFO, OnLVGetDispInfo)

		CHAIN_MSG_MAP(CUpdateUI<CMainFrame>)
		CHAIN_MSG_MAP(CFrameWindowImpl<CMainFrame>)
	END_MSG_MAP()

// Handler prototypes (uncomment arguments if needed):
//	LRESULT MessageHandler(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
//	LRESULT CommandHandler(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
//	LRESULT NotifyHandler(int /*idCtrl*/, LPNMHDR /*pnmh*/, BOOL& /*bHandled*/)

   LRESULT OnDestroy(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& bHandled)
	{
		bHandled = 0;
		// save position of dock bars

#ifdef DOCK_DUAL_REGINI
		CRegKeyEx key(GetRegKeyEx(0, 0));
		m_dock.SaveBarState(key);
#else
		m_dock.SaveBarState(m_strRegistryKey);
#endif

		return 0;
	}

	LRESULT OnCreate(UINT /*uMsg*/, WPARAM /*wParam*/, LPARAM /*lParam*/, BOOL& /*bHandled*/)
	{
		HWND hWndToolBar = CreateSimpleToolBarCtrl(m_hWnd, IDR_MAINFRAME, FALSE, ATL_SIMPLE_TOOLBAR_PANE_STYLE);

		CreateSimpleReBar(ATL_SIMPLE_REBAR_NOBORDER_STYLE);
		AddSimpleReBarBand(hWndToolBar);

		CreateSimpleStatusBar();

		// Initiate docking framework
		m_hWndClient = m_dock.Create(m_hWnd, rcDefault);

		// real client is subsumed by the docking framework
		HWND hWndView = m_view.Create(m_hWndClient, rcDefault, NULL, WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_HSCROLL | WS_VSCROLL | ES_AUTOHSCROLL | ES_AUTOVSCROLL | ES_MULTILINE | ES_NOHIDESEL, WS_EX_CLIENTEDGE);
      m_dock.SetClient(hWndView);

		// add docked panes (parent doesn't matter much)
		m_bar1.Create(m_hWnd, rcDefault, _T("View1"), ATL_SIMPLE_DOCKVIEW_STYLE|ES_MULTILINE, WS_EX_CLIENTEDGE, ATL_IDW_DOCKPANE_FIRST+1);
      m_bar2.Create(m_hWnd, rcDefault, _T("View2"), ATL_SIMPLE_DOCKVIEW_STYLE|ES_MULTILINE, WS_EX_CLIENTEDGE, ATL_IDW_DOCKPANE_FIRST+2);
      m_bar3.Create(m_hWnd, rcDefault, _T("View3"), ATL_SIMPLE_DOCKVIEW_STYLE|ES_MULTILINE, WS_EX_CLIENTEDGE, ATL_IDW_DOCKPANE_FIRST+3);
		m_bar4.Create(m_hWnd, rcDefault, _T("View4"), ATL_SIMPLE_DOCKVIEW_STYLE, WS_EX_CLIENTEDGE, ATL_IDW_DOCKPANE_FIRST);

      m_dock.AddWindow(m_bar1, DCK_TITLETEXT|DCK_GRADIENTEXT|DCK_MANUALSIDE|DCK_LEFTBAR);
      m_dock.AddWindow(m_bar2);
      m_dock.AddWindow(m_bar3, DCK_TITLETEXT);
		m_dock.AddWindow(m_bar4, DCK_TITLETEXT|DCK_GRADIENTEXT);
		//m_bar4.AddItem(0, 0, _T("item 0"));

		// default docked pane arrangement
		DOCK_DEFBARSTATE state[] = {
			{ATL_IDW_DOCKPANE_FIRST+1, DOCK_LEFT, 0},
			{ATL_IDW_DOCKPANE_FIRST+2, DOCK_LEFT, 1},
			{ATL_IDW_DOCKPANE_FIRST+3, DOCK_RIGHT, 1},
			{ATL_IDW_DOCKPANE_FIRST, DOCK_BOTTOM, 1}
		};

		m_strRegistryKey = _T("software\\test");
#ifdef DOCK_DUAL_REGINI
		SetRegistryKey(m_strRegistryKey);
		SetIniFile(_T("C:\\test.ini"));
		CRegKeyEx key(GetRegKeyEx(0));
		m_dock.LoadBarState(key, state);
#else
		m_dock.LoadBarState(m_strRegistryKey, state);
#endif

		PostMessage(WM_COMMAND, ID_FILE_PRINT); // adds element

		UIAddToolBar(hWndToolBar);
		UISetCheck(ID_VIEW_TOOLBAR, 1);
		UISetCheck(ID_VIEW_STATUS_BAR, 1);

		// register object for message filtering and idle updates
		CMessageLoop* pLoop = _Module.GetMessageLoop();
		ATLASSERT(pLoop != NULL);
		pLoop->AddMessageFilter(this);
		pLoop->AddIdleHandler(this);

		return 0;
	}

	// test notifications
	LRESULT OnLVGetDispInfo(int /*idCtrl*/, LPNMHDR pnmh, BOOL& bHandled)
	{
		ATLASSERT(pnmh->hwndFrom == m_bar4);
		LV_ITEM& it = ((LV_DISPINFO*)pnmh)->item;

		CWindow wnd = m_bar4.GetParent(); // always the original frame!?
		if(it.mask & LVIF_TEXT) {
			static TCHAR buf[10];
			wsprintf(buf, _T("item %d"), it.iItem);
			it.pszText = buf;
			it.mask |= LVIF_DI_SETITEM;
		}
		else
			bHandled = FALSE;

		return 0;
	}

	LRESULT OnAddItem(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		LVITEM lvItem;
		lvItem.mask = LVIF_TEXT;
		lvItem.iItem = m_bar4.GetItemCount();
		lvItem.iSubItem = 0;
		lvItem.pszText = LPSTR_TEXTCALLBACK;
		m_bar4.InsertItem(&lvItem);

		return 0;
	}

	LRESULT OnFileExit(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		PostMessage(WM_CLOSE);
		return 0;
	}

	LRESULT OnFileNew(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		// TODO: add code to initialize document

		return 0;
	}

	LRESULT OnViewToolBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		static BOOL bVisible = TRUE;	// initially visible
		bVisible = !bVisible;
		CReBarCtrl rebar = m_hWndToolBar;
		int nBandIndex = rebar.IdToIndex(ATL_IDW_BAND_FIRST);	// toolbar is first 1st band
		rebar.ShowBand(nBandIndex, bVisible);
		UISetCheck(ID_VIEW_TOOLBAR, bVisible);
		UpdateLayout();
		return 0;
	}

	LRESULT OnViewStatusBar(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		BOOL bVisible = !::IsWindowVisible(m_hWndStatusBar);
		::ShowWindow(m_hWndStatusBar, bVisible ? SW_SHOWNOACTIVATE : SW_HIDE);
		UISetCheck(ID_VIEW_STATUS_BAR, bVisible);
		UpdateLayout();
		return 0;
	}

	
   LRESULT OnLockDocks(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		m_dock.Lock(!m_dock.IsLocked());
		return 0;
	}

   LRESULT OnSwitchDock(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		// this assumes that some dockpane is focused
		m_dock.FocusNextPane(ID_NEXT_PANE==wID);

		return 0;
	}

   LRESULT OnViewBar(WORD /*wNotifyCode*/, WORD wID, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
   {
      HWND hWnd;
      switch( wID ) {
      case ID_VIEW_BAR1: hWnd = m_bar1; break;
      case ID_VIEW_BAR2: hWnd = m_bar2; break;
      case ID_VIEW_BAR3: hWnd = m_bar3; break;
      case ID_VIEW_BAR4: hWnd = m_bar4; break;
      default: return 0;
      }

		m_dock.HideWindow(hWnd, ::IsWindowVisible(hWnd));
      return 0;
   }

	LRESULT OnAppAbout(WORD /*wNotifyCode*/, WORD /*wID*/, HWND /*hWndCtl*/, BOOL& /*bHandled*/)
	{
		CAboutDlg dlg;
		dlg.DoModal();
		return 0;
	}
};

/////////////////////////////////////////////////////////////////////////////

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_MAINFRM_H__5DBB058E_0908_442B_9ED4_DECBDE98E4CD__INCLUDED_)
